@@ -22,6 +22,7 @@
 #include <QVariant>
 #include <QFileDialog>
 #include "window-basic-main.hpp"
+#include "window-basic-auto-config.hpp"
 #include "window-namedialog.hpp"
 #include "qt-wrappers.hpp"
 
@@ -30,13 +31,13 @@ extern void DuplicateCurrentCookieProfile(ConfigFile &config);
 extern void CheckExistingCookieId();
 extern void DeleteCookies();
 
-void EnumProfiles(std::function<bool (const char *, const char *)> &&cb)
+void EnumProfiles(std::function<bool(const char *, const char *)> &&cb)
 {
 	char path[512];
 	os_glob_t *glob;
 
 	int ret = GetConfigPath(path, sizeof(path),
-			"obs-studio/basic/profiles/*");
+				"obs-studio/basic/profiles/*");
 	if (ret <= 0) {
 		blog(LOG_WARNING, "Failed to get profiles config path");
 		return;
@@ -54,8 +55,7 @@ void EnumProfiles(std::function<bool (const char *, const char *)> &&cb)
 		if (!glob->gl_pathv[i].directory)
 			continue;
 
-		if (strcmp(dirName,  ".") == 0 ||
-		    strcmp(dirName, "..") == 0)
+		if (strcmp(dirName, ".") == 0 || strcmp(dirName, "..") == 0)
 			continue;
 
 		std::string file = filePath;
@@ -77,13 +77,13 @@ void EnumProfiles(std::function<bool (const char *, const char *)> &&cb)
 	os_globfree(glob);
 }
 
-static bool ProfileExists(const char *findName)
+static bool GetProfileDir(const char *findName, const char *&profileDir)
 {
 	bool found = false;
-	auto func = [&](const char *name, const char*)
-	{
+	auto func = [&](const char *name, const char *path) {
 		if (strcmp(name, findName) == 0) {
 			found = true;
+			profileDir = strrchr(path, '/') + 1;
 			return false;
 		}
 		return true;
@@ -93,37 +93,64 @@ static bool ProfileExists(const char *findName)
 	return found;
 }
 
-static bool GetProfileName(QWidget *parent, std::string &name,
-		std::string &file, const char *title, const char *text,
-		const char *oldName = nullptr)
+static bool ProfileExists(const char *findName)
 {
-	char path[512];
-	int ret;
+	const char *profileDir = nullptr;
+	return GetProfileDir(findName, profileDir);
+}
 
+static bool AskForProfileName(QWidget *parent, std::string &name,
+			      const char *title, const char *text,
+			      const bool showWizard, bool &wizardChecked,
+			      const char *oldName = nullptr)
+{
 	for (;;) {
-		bool success = NameDialog::AskForName(parent, title, text,
-				name, QT_UTF8(oldName));
+		bool success = false;
+
+		if (showWizard) {
+			success = NameDialog::AskForNameWithOption(
+				parent, title, text, name,
+				QTStr("AddProfile.WizardCheckbox"),
+				wizardChecked, QT_UTF8(oldName));
+		} else {
+			success = NameDialog::AskForName(
+				parent, title, text, name, QT_UTF8(oldName));
+		}
+
 		if (!success) {
 			return false;
 		}
 		if (name.empty()) {
-			OBSMessageBox::information(parent,
-					QTStr("NoNameEntered.Title"),
-					QTStr("NoNameEntered.Text"));
+			OBSMessageBox::warning(parent,
+					       QTStr("NoNameEntered.Title"),
+					       QTStr("NoNameEntered.Text"));
 			continue;
 		}
 		if (ProfileExists(name.c_str())) {
-			OBSMessageBox::information(parent,
-					QTStr("NameExists.Title"),
-					QTStr("NameExists.Text"));
+			OBSMessageBox::warning(parent,
+					       QTStr("NameExists.Title"),
+					       QTStr("NameExists.Text"));
 			continue;
 		}
 		break;
 	}
+	return true;
+}
 
-	if (!GetFileSafeName(name.c_str(), file)) {
+static bool FindSafeProfileDirName(const std::string &profileName,
+				   std::string &dirName)
+{
+	char path[512];
+	int ret;
+
+	if (ProfileExists(profileName.c_str())) {
+		blog(LOG_WARNING, "Profile '%s' exists", profileName.c_str());
+		return false;
+	}
+
+	if (!GetFileSafeName(profileName.c_str(), dirName)) {
 		blog(LOG_WARNING, "Failed to create safe file name for '%s'",
-				name.c_str());
+		     profileName.c_str());
 		return false;
 	}
 
@@ -133,15 +160,15 @@ static bool GetProfileName(QWidget *parent, std::string &name,
 		return false;
 	}
 
-	file.insert(0, path);
+	dirName.insert(0, path);
 
-	if (!GetClosestUnusedFileName(file, nullptr)) {
+	if (!GetClosestUnusedFileName(dirName, nullptr)) {
 		blog(LOG_WARNING, "Failed to get closest file name for %s",
-				file.c_str());
+		     dirName.c_str());
 		return false;
 	}
 
-	file.erase(0, ret);
+	dirName.erase(0, ret);
 	return true;
 }
 
@@ -170,13 +197,14 @@ static bool CopyProfile(const char *fromPartial, const char *to)
 		if (glob->gl_pathv[i].directory)
 			continue;
 
-		ret = snprintf(path, sizeof(path), "%s/%s",
-				to, strrchr(filePath, '/') + 1);
+		ret = snprintf(path, sizeof(path), "%s/%s", to,
+			       strrchr(filePath, '/') + 1);
 		if (ret > 0) {
 			if (os_copyfile(filePath, path) != 0) {
-				blog(LOG_WARNING, "CopyProfile: Failed to "
-				                  "copy file %s to %s",
-				                  filePath, path);
+				blog(LOG_WARNING,
+				     "CopyProfile: Failed to "
+				     "copy file %s to %s",
+				     filePath, path);
 			}
 		}
 	}
@@ -186,23 +214,72 @@ static bool CopyProfile(const char *fromPartial, const char *to)
 	return true;
 }
 
+static bool ProfileNeedsRestart(config_t *newConfig, QString &settings)
+{
+	OBSBasic *main = OBSBasic::Get();
+
+	const char *oldSpeakers =
+		config_get_string(main->Config(), "Audio", "ChannelSetup");
+	uint oldSampleRate =
+		config_get_uint(main->Config(), "Audio", "SampleRate");
+
+	const char *newSpeakers =
+		config_get_string(newConfig, "Audio", "ChannelSetup");
+	uint newSampleRate = config_get_uint(newConfig, "Audio", "SampleRate");
+
+	auto appendSetting = [&settings](const char *name) {
+		settings += QStringLiteral("\n") + QTStr(name);
+	};
+
+	bool result = false;
+	if (oldSpeakers != NULL && newSpeakers != NULL) {
+		result = strcmp(oldSpeakers, newSpeakers) != 0;
+		appendSetting("Basic.Settings.Audio.Channels");
+	}
+	if (oldSampleRate != 0 && newSampleRate != 0) {
+		result |= oldSampleRate != newSampleRate;
+		appendSetting("Basic.Settings.Audio.SampleRate");
+	}
+
+	return result;
+}
+
 bool OBSBasic::AddProfile(bool create_new, const char *title, const char *text,
-		const char *init_text, bool rename)
+			  const char *init_text, bool rename)
+{
+	std::string name;
+
+	bool showWizardChecked = config_get_bool(App()->GlobalConfig(), "Basic",
+						 "ConfigOnNewProfile");
+
+	if (!AskForProfileName(this, name, title, text, create_new,
+			       showWizardChecked, init_text))
+		return false;
+
+	return CreateProfile(name, create_new, showWizardChecked, rename);
+}
+
+bool OBSBasic::CreateProfile(const std::string &newName, bool create_new,
+			     bool showWizardChecked, bool rename)
 {
-	std::string newName;
 	std::string newDir;
 	std::string newPath;
 	ConfigFile config;
 
-	if (!GetProfileName(this, newName, newDir, title, text, init_text))
+	if (!FindSafeProfileDirName(newName, newDir))
 		return false;
 
-	std::string curDir = config_get_string(App()->GlobalConfig(),
-			"Basic", "ProfileDir");
+	if (create_new) {
+		config_set_bool(App()->GlobalConfig(), "Basic",
+				"ConfigOnNewProfile", showWizardChecked);
+	}
+
+	std::string curDir =
+		config_get_string(App()->GlobalConfig(), "Basic", "ProfileDir");
 
 	char baseDir[512];
 	int ret = GetConfigPath(baseDir, sizeof(baseDir),
-			"obs-studio/basic/profiles/");
+				"obs-studio/basic/profiles/");
 	if (ret <= 0) {
 		blog(LOG_WARNING, "Failed to get profiles config path");
 		return false;
@@ -213,7 +290,7 @@ bool OBSBasic::AddProfile(bool create_new, const char *title, const char *text,
 
 	if (os_mkdir(newPath.c_str()) < 0) {
 		blog(LOG_WARNING, "Failed to create profile directory '%s'",
-				newDir.c_str());
+		     newDir.c_str());
 		return false;
 	}
 
@@ -224,14 +301,17 @@ bool OBSBasic::AddProfile(bool create_new, const char *title, const char *text,
 
 	if (config.Open(newPath.c_str(), CONFIG_OPEN_ALWAYS) != 0) {
 		blog(LOG_ERROR, "Failed to open new config file '%s'",
-				newDir.c_str());
+		     newDir.c_str());
 		return false;
 	}
 
+	if (api && !rename)
+		api->on_event(OBS_FRONTEND_EVENT_PROFILE_CHANGING);
+
 	config_set_string(App()->GlobalConfig(), "Basic", "Profile",
-			newName.c_str());
+			  newName.c_str());
 	config_set_string(App()->GlobalConfig(), "Basic", "ProfileDir",
-			newDir.c_str());
+			  newDir.c_str());
 
 	Auth::Save();
 	if (create_new) {
@@ -253,19 +333,41 @@ bool OBSBasic::AddProfile(bool create_new, const char *title, const char *text,
 		ResetProfileData();
 
 	blog(LOG_INFO, "Created profile '%s' (%s, %s)", newName.c_str(),
-			create_new ? "clean" : "duplicate", newDir.c_str());
+	     create_new ? "clean" : "duplicate", newDir.c_str());
 	blog(LOG_INFO, "------------------------------------------------");
 
 	config_save_safe(App()->GlobalConfig(), "tmp", nullptr);
 	UpdateTitleBar();
+	UpdateVolumeControlsDecayRate();
 
-	if (api) {
+	Auth::Load();
+
+	// Run auto configuration setup wizard when a new profile is made to assist
+	// setting up blank settings
+	if (create_new && showWizardChecked) {
+		AutoConfig wizard(this);
+		wizard.setModal(true);
+		wizard.show();
+		wizard.exec();
+	}
+
+	if (api && !rename) {
 		api->on_event(OBS_FRONTEND_EVENT_PROFILE_LIST_CHANGED);
 		api->on_event(OBS_FRONTEND_EVENT_PROFILE_CHANGED);
 	}
 	return true;
 }
 
+bool OBSBasic::NewProfile(const QString &name)
+{
+	return CreateProfile(name.toStdString(), true, false, false);
+}
+
+bool OBSBasic::DuplicateProfile(const QString &name)
+{
+	return CreateProfile(name.toStdString(), false, false, false);
+}
+
 void OBSBasic::DeleteProfile(const char *profileName, const char *profileDir)
 {
 	char profilePath[512];
@@ -280,14 +382,14 @@ void OBSBasic::DeleteProfile(const char *profileName, const char *profileDir)
 	ret = snprintf(profilePath, 512, "%s/%s/*", basePath, profileDir);
 	if (ret <= 0) {
 		blog(LOG_WARNING, "Failed to get path for profile dir '%s'",
-				profileDir);
+		     profileDir);
 		return;
 	}
 
 	os_glob_t *glob;
 	if (os_glob(profilePath, 0, &glob) != 0) {
 		blog(LOG_WARNING, "Failed to glob profile dir '%s'",
-				profileDir);
+		     profileDir);
 		return;
 	}
 
@@ -305,21 +407,50 @@ void OBSBasic::DeleteProfile(const char *profileName, const char *profileDir)
 	ret = snprintf(profilePath, 512, "%s/%s", basePath, profileDir);
 	if (ret <= 0) {
 		blog(LOG_WARNING, "Failed to get path for profile dir '%s'",
-				profileDir);
+		     profileDir);
 		return;
 	}
 
 	os_rmdir(profilePath);
 
 	blog(LOG_INFO, "------------------------------------------------");
-	blog(LOG_INFO, "Removed profile '%s' (%s)",
-			profileName, profileDir);
+	blog(LOG_INFO, "Removed profile '%s' (%s)", profileName, profileDir);
 	blog(LOG_INFO, "------------------------------------------------");
 }
 
+void OBSBasic::DeleteProfile(const QString &profileName)
+{
+	std::string name = profileName.toStdString();
+	const char *curName =
+		config_get_string(App()->GlobalConfig(), "Basic", "Profile");
+
+	if (strcmp(curName, name.c_str()) == 0) {
+		on_actionRemoveProfile_triggered(true);
+		return;
+	}
+
+	const char *profileDir = nullptr;
+	if (!GetProfileDir(name.c_str(), profileDir)) {
+		blog(LOG_WARNING, "Profile '%s' not found", name.c_str());
+		return;
+	}
+
+	if (!profileDir) {
+		blog(LOG_WARNING, "Failed to get profile dir for profile '%s'",
+		     name.c_str());
+		return;
+	}
+
+	DeleteProfile(name.c_str(), profileDir);
+	RefreshProfiles();
+	config_save_safe(App()->GlobalConfig(), "tmp", nullptr);
+	if (api)
+		api->on_event(OBS_FRONTEND_EVENT_PROFILE_LIST_CHANGED);
+}
+
 void OBSBasic::RefreshProfiles()
 {
-	QList<QAction*> menuActions = ui->profileMenu->actions();
+	QList<QAction *> menuActions = ui->profileMenu->actions();
 	int count = 0;
 
 	for (int i = 0; i < menuActions.count(); i++) {
@@ -328,17 +459,16 @@ void OBSBasic::RefreshProfiles()
 			delete menuActions[i];
 	}
 
-	const char *curName = config_get_string(App()->GlobalConfig(),
-			"Basic", "Profile");
+	const char *curName =
+		config_get_string(App()->GlobalConfig(), "Basic", "Profile");
 
-	auto addProfile = [&](const char *name, const char *path)
-	{
+	auto addProfile = [&](const char *name, const char *path) {
 		std::string file = strrchr(path, '/') + 1;
 
 		QAction *action = new QAction(QT_UTF8(name), this);
 		action->setProperty("file_name", QT_UTF8(path));
-		connect(action, &QAction::triggered,
-				this, &OBSBasic::ChangeProfile);
+		connect(action, &QAction::triggered, this,
+			&OBSBasic::ChangeProfile);
 		action->setCheckable(true);
 
 		action->setChecked(strcmp(name, curName) == 0);
@@ -363,17 +493,17 @@ void OBSBasic::ResetProfileData()
 	CreateHotkeys();
 
 	/* load audio monitoring */
-#if defined(_WIN32) || defined(__APPLE__) || HAVE_PULSEAUDIO
-	const char *device_name = config_get_string(basicConfig, "Audio",
-			"MonitoringDeviceName");
-	const char *device_id = config_get_string(basicConfig, "Audio",
-			"MonitoringDeviceId");
+	if (obs_audio_monitoring_available()) {
+		const char *device_name = config_get_string(
+			basicConfig, "Audio", "MonitoringDeviceName");
+		const char *device_id = config_get_string(basicConfig, "Audio",
+							  "MonitoringDeviceId");
 
-	obs_set_audio_monitoring_device(device_name, device_id);
+		obs_set_audio_monitoring_device(device_name, device_id);
 
-	blog(LOG_INFO, "Audio monitoring device:\n\tname: %s\n\tid: %s",
-			device_name, device_id);
-#endif
+		blog(LOG_INFO, "Audio monitoring device:\n\tname: %s\n\tid: %s",
+		     device_name, device_id);
+	}
 }
 
 void OBSBasic::on_actionNewProfile_triggered()
@@ -388,38 +518,36 @@ void OBSBasic::on_actionDupProfile_triggered()
 
 void OBSBasic::on_actionRenameProfile_triggered()
 {
-	std::string curDir = config_get_string(App()->GlobalConfig(),
-			"Basic", "ProfileDir");
-	std::string curName = config_get_string(App()->GlobalConfig(),
-			"Basic", "Profile");
+	std::string curDir =
+		config_get_string(App()->GlobalConfig(), "Basic", "ProfileDir");
+	std::string curName =
+		config_get_string(App()->GlobalConfig(), "Basic", "Profile");
 
 	/* Duplicate and delete in case there are any issues in the process */
 	bool success = AddProfile(false, Str("RenameProfile.Title"),
-			Str("AddProfile.Text"), curName.c_str(), true);
+				  Str("AddProfile.Text"), curName.c_str(),
+				  true);
 	if (success) {
 		DeleteProfile(curName.c_str(), curDir.c_str());
 		RefreshProfiles();
 	}
 
-	if (api) {
-		api->on_event(OBS_FRONTEND_EVENT_PROFILE_LIST_CHANGED);
-		api->on_event(OBS_FRONTEND_EVENT_PROFILE_CHANGED);
-	}
+	if (api)
+		api->on_event(OBS_FRONTEND_EVENT_PROFILE_RENAMED);
 }
 
-void OBSBasic::on_actionRemoveProfile_triggered()
+void OBSBasic::on_actionRemoveProfile_triggered(bool skipConfirmation)
 {
 	std::string newName;
 	std::string newPath;
 	ConfigFile config;
 
-	std::string oldDir = config_get_string(App()->GlobalConfig(),
-			"Basic", "ProfileDir");
-	std::string oldName = config_get_string(App()->GlobalConfig(),
-			"Basic", "Profile");
+	std::string oldDir =
+		config_get_string(App()->GlobalConfig(), "Basic", "ProfileDir");
+	std::string oldName =
+		config_get_string(App()->GlobalConfig(), "Basic", "Profile");
 
-	auto cb = [&](const char *name, const char *filePath)
-	{
+	auto cb = [&](const char *name, const char *filePath) {
 		if (strcmp(oldName.c_str(), name) != 0) {
 			newName = name;
 			newPath = filePath;
@@ -435,31 +563,39 @@ void OBSBasic::on_actionRemoveProfile_triggered()
 	if (newPath.empty())
 		return;
 
-	QString text = QTStr("ConfirmRemove.Text");
-	text.replace("$1", QT_UTF8(oldName.c_str()));
+	if (!skipConfirmation) {
+		QString text = QTStr("ConfirmRemove.Text")
+				       .arg(QT_UTF8(oldName.c_str()));
 
-	QMessageBox::StandardButton button = OBSMessageBox::question(this,
-			QTStr("ConfirmRemove.Title"), text);
-	if (button == QMessageBox::No)
-		return;
+		QMessageBox::StandardButton button = OBSMessageBox::question(
+			this, QTStr("ConfirmRemove.Title"), text);
+		if (button == QMessageBox::No)
+			return;
+	}
 
 	size_t newPath_len = newPath.size();
 	newPath += "/basic.ini";
 
 	if (config.Open(newPath.c_str(), CONFIG_OPEN_ALWAYS) != 0) {
 		blog(LOG_ERROR, "ChangeProfile: Failed to load file '%s'",
-				newPath.c_str());
+		     newPath.c_str());
 		return;
 	}
 
+	if (api)
+		api->on_event(OBS_FRONTEND_EVENT_PROFILE_CHANGING);
+
 	newPath.resize(newPath_len);
 
 	const char *newDir = strrchr(newPath.c_str(), '/') + 1;
 
 	config_set_string(App()->GlobalConfig(), "Basic", "Profile",
-			newName.c_str());
-	config_set_string(App()->GlobalConfig(), "Basic", "ProfileDir",
-			newDir);
+			  newName.c_str());
+	config_set_string(App()->GlobalConfig(), "Basic", "ProfileDir", newDir);
+
+	QString settingsRequiringRestart;
+	bool needsRestart =
+		ProfileNeedsRestart(config, settingsRequiringRestart);
 
 	Auth::Save();
 	auth.reset();
@@ -474,11 +610,12 @@ void OBSBasic::on_actionRemoveProfile_triggered()
 	RefreshProfiles();
 	config_save_safe(App()->GlobalConfig(), "tmp", nullptr);
 
-	blog(LOG_INFO, "Switched to profile '%s' (%s)",
-			newName.c_str(), newDir);
+	blog(LOG_INFO, "Switched to profile '%s' (%s)", newName.c_str(),
+	     newDir);
 	blog(LOG_INFO, "------------------------------------------------");
 
 	UpdateTitleBar();
+	UpdateVolumeControlsDecayRate();
 
 	Auth::Load();
 
@@ -486,6 +623,18 @@ void OBSBasic::on_actionRemoveProfile_triggered()
 		api->on_event(OBS_FRONTEND_EVENT_PROFILE_LIST_CHANGED);
 		api->on_event(OBS_FRONTEND_EVENT_PROFILE_CHANGED);
 	}
+
+	if (needsRestart) {
+		QMessageBox::StandardButton button = OBSMessageBox::question(
+			this, QTStr("Restart"),
+			QTStr("LoadProfileNeedsRestart")
+				.arg(settingsRequiringRestart));
+
+		if (button == QMessageBox::Yes) {
+			restart = true;
+			close();
+		}
+	}
 }
 
 void OBSBasic::on_actionImportProfile_triggered()
@@ -500,35 +649,33 @@ void OBSBasic::on_actionImportProfile_triggered()
 		return;
 	}
 
-	QString dir = QFileDialog::getExistingDirectory(
-			this,
-			QTStr("Basic.MainMenu.Profile.Import"),
-			home,
-			QFileDialog::ShowDirsOnly |
-			QFileDialog::DontResolveSymlinks);
+	QString dir = SelectDirectory(
+		this, QTStr("Basic.MainMenu.Profile.Import"), home);
 
 	if (!dir.isEmpty() && !dir.isNull()) {
 		QString inputPath = QString::fromUtf8(path);
 		QFileInfo finfo(dir);
 		QString directory = finfo.fileName();
 		QString profileDir = inputPath + directory;
-		QDir folder(profileDir);
 
-		if (!folder.exists()) {
-			folder.mkpath(profileDir);
-			QFile::copy(dir + "/basic.ini",
-					profileDir + "/basic.ini");
-			QFile::copy(dir + "/service.json",
-					profileDir + "/service.json");
-			QFile::copy(dir + "/streamEncoder.json",
-					profileDir + "/streamEncoder.json");
-			QFile::copy(dir + "/recordEncoder.json",
-					profileDir + "/recordEncoder.json");
-			RefreshProfiles();
+		if (ProfileExists(directory.toStdString().c_str())) {
+			OBSMessageBox::warning(
+				this, QTStr("Basic.MainMenu.Profile.Import"),
+				QTStr("Basic.MainMenu.Profile.Exists"));
+		} else if (os_mkdir(profileDir.toStdString().c_str()) < 0) {
+			blog(LOG_WARNING,
+			     "Failed to create profile directory '%s'",
+			     directory.toStdString().c_str());
 		} else {
-			OBSMessageBox::information(this,
-					QTStr("Basic.MainMenu.Profile.Import"),
-					QTStr("Basic.MainMenu.Profile.Exists"));
+			QFile::copy(dir + "/basic.ini",
+				    profileDir + "/basic.ini");
+			QFile::copy(dir + "/service.json",
+				    profileDir + "/service.json");
+			QFile::copy(dir + "/streamEncoder.json",
+				    profileDir + "/streamEncoder.json");
+			QFile::copy(dir + "/recordEncoder.json",
+				    profileDir + "/recordEncoder.json");
+			RefreshProfiles();
 		}
 	}
 }
@@ -539,9 +686,8 @@ void OBSBasic::on_actionExportProfile_triggered()
 
 	QString home = QDir::homePath();
 
-	QString currentProfile =
-		QString::fromUtf8(config_get_string(App()->GlobalConfig(),
-		"Basic", "ProfileDir"));
+	QString currentProfile = QString::fromUtf8(config_get_string(
+		App()->GlobalConfig(), "Basic", "ProfileDir"));
 
 	int ret = GetConfigPath(path, 512, "obs-studio/basic/profiles/");
 	if (ret <= 0) {
@@ -549,12 +695,8 @@ void OBSBasic::on_actionExportProfile_triggered()
 		return;
 	}
 
-	QString dir = QFileDialog::getExistingDirectory(
-			this,
-			QTStr("Basic.MainMenu.Profile.Export"),
-			home,
-			QFileDialog::ShowDirsOnly |
-			QFileDialog::DontResolveSymlinks);
+	QString dir = SelectDirectory(
+		this, QTStr("Basic.MainMenu.Profile.Export"), home);
 
 	if (!dir.isEmpty() && !dir.isNull()) {
 		QString outputDir = dir + "/" + currentProfile;
@@ -571,26 +713,28 @@ void OBSBasic::on_actionExportProfile_triggered()
 				QFile::remove(outputDir + "/service.json");
 
 			if (QFile::exists(outputDir + "/streamEncoder.json"))
-				QFile::remove(outputDir + "/streamEncoder.json");
+				QFile::remove(outputDir +
+					      "/streamEncoder.json");
 
 			if (QFile::exists(outputDir + "/recordEncoder.json"))
-				QFile::remove(outputDir + "/recordEncoder.json");
+				QFile::remove(outputDir +
+					      "/recordEncoder.json");
 		}
 
 		QFile::copy(inputPath + currentProfile + "/basic.ini",
-				outputDir + "/basic.ini");
+			    outputDir + "/basic.ini");
 		QFile::copy(inputPath + currentProfile + "/service.json",
-				outputDir + "/service.json");
+			    outputDir + "/service.json");
 		QFile::copy(inputPath + currentProfile + "/streamEncoder.json",
-				outputDir + "/streamEncoder.json");
+			    outputDir + "/streamEncoder.json");
 		QFile::copy(inputPath + currentProfile + "/recordEncoder.json",
-				outputDir + "/recordEncoder.json");
+			    outputDir + "/recordEncoder.json");
 	}
 }
 
 void OBSBasic::ChangeProfile()
 {
-	QAction *action = reinterpret_cast<QAction*>(sender());
+	QAction *action = reinterpret_cast<QAction *>(sender());
 	ConfigFile config;
 	std::string path;
 
@@ -601,8 +745,8 @@ void OBSBasic::ChangeProfile()
 	if (path.empty())
 		return;
 
-	const char *oldName = config_get_string(App()->GlobalConfig(),
-			"Basic", "Profile");
+	const char *oldName =
+		config_get_string(App()->GlobalConfig(), "Basic", "Profile");
 	if (action->text().compare(QT_UTF8(oldName)) == 0) {
 		action->setChecked(true);
 		return;
@@ -613,18 +757,24 @@ void OBSBasic::ChangeProfile()
 
 	if (config.Open(path.c_str(), CONFIG_OPEN_ALWAYS) != 0) {
 		blog(LOG_ERROR, "ChangeProfile: Failed to load file '%s'",
-				path.c_str());
+		     path.c_str());
 		return;
 	}
 
+	if (api)
+		api->on_event(OBS_FRONTEND_EVENT_PROFILE_CHANGING);
+
 	path.resize(path_len);
 
 	const char *newName = config_get_string(config, "General", "Name");
 	const char *newDir = strrchr(path.c_str(), '/') + 1;
 
+	QString settingsRequiringRestart;
+	bool needsRestart =
+		ProfileNeedsRestart(config, settingsRequiringRestart);
+
 	config_set_string(App()->GlobalConfig(), "Basic", "Profile", newName);
-	config_set_string(App()->GlobalConfig(), "Basic", "ProfileDir",
-			newDir);
+	config_set_string(App()->GlobalConfig(), "Basic", "ProfileDir", newDir);
 
 	Auth::Save();
 	auth.reset();
@@ -637,28 +787,45 @@ void OBSBasic::ChangeProfile()
 	RefreshProfiles();
 	config_save_safe(App()->GlobalConfig(), "tmp", nullptr);
 	UpdateTitleBar();
+	UpdateVolumeControlsDecayRate();
 
 	Auth::Load();
 
 	CheckForSimpleModeX264Fallback();
 
-	blog(LOG_INFO, "Switched to profile '%s' (%s)",
-			newName, newDir);
+	blog(LOG_INFO, "Switched to profile '%s' (%s)", newName, newDir);
 	blog(LOG_INFO, "------------------------------------------------");
 
 	if (api)
 		api->on_event(OBS_FRONTEND_EVENT_PROFILE_CHANGED);
+
+	if (needsRestart) {
+		QMessageBox::StandardButton button = OBSMessageBox::question(
+			this, QTStr("Restart"),
+			QTStr("LoadProfileNeedsRestart")
+				.arg(settingsRequiringRestart));
+
+		if (button == QMessageBox::Yes) {
+			restart = true;
+			close();
+		}
+	}
 }
 
 void OBSBasic::CheckForSimpleModeX264Fallback()
 {
-	const char *curStreamEncoder = config_get_string(basicConfig,
-			"SimpleOutput", "StreamEncoder");
-	const char *curRecEncoder = config_get_string(basicConfig,
-			"SimpleOutput", "RecEncoder");
+	const char *curStreamEncoder =
+		config_get_string(basicConfig, "SimpleOutput", "StreamEncoder");
+	const char *curRecEncoder =
+		config_get_string(basicConfig, "SimpleOutput", "RecEncoder");
 	bool qsv_supported = false;
 	bool amd_supported = false;
 	bool nve_supported = false;
+#ifdef ENABLE_HEVC
+	bool amd_hevc_supported = false;
+	bool nve_hevc_supported = false;
+#endif
+	bool apple_supported = false;
 	bool changed = false;
 	size_t idx = 0;
 	const char *id;
@@ -670,10 +837,19 @@ void OBSBasic::CheckForSimpleModeX264Fallback()
 			qsv_supported = true;
 		else if (strcmp(id, "ffmpeg_nvenc") == 0)
 			nve_supported = true;
+#ifdef ENABLE_HEVC
+		else if (strcmp(id, "h265_texture_amf") == 0)
+			amd_hevc_supported = true;
+		else if (strcmp(id, "ffmpeg_hevc_nvenc") == 0)
+			nve_hevc_supported = true;
+#endif
+		else if (strcmp(id,
+				"com.apple.videotoolbox.videoencoder.ave.avc") ==
+			 0)
+			apple_supported = true;
 	}
 
-	auto CheckEncoder = [&] (const char *&name)
-	{
+	auto CheckEncoder = [&](const char *&name) {
 		if (strcmp(name, SIMPLE_ENCODER_QSV) == 0) {
 			if (!qsv_supported) {
 				changed = true;
@@ -686,25 +862,49 @@ void OBSBasic::CheckForSimpleModeX264Fallback()
 				name = SIMPLE_ENCODER_X264;
 				return false;
 			}
+		} else if (strcmp(name, SIMPLE_ENCODER_NVENC_AV1) == 0) {
+			if (!nve_supported) {
+				changed = true;
+				name = SIMPLE_ENCODER_X264;
+				return false;
+			}
+#ifdef ENABLE_HEVC
+		} else if (strcmp(name, SIMPLE_ENCODER_AMD_HEVC) == 0) {
+			if (!amd_hevc_supported) {
+				changed = true;
+				name = SIMPLE_ENCODER_X264;
+				return false;
+			}
+		} else if (strcmp(name, SIMPLE_ENCODER_NVENC_HEVC) == 0) {
+			if (!nve_hevc_supported) {
+				changed = true;
+				name = SIMPLE_ENCODER_X264;
+				return false;
+			}
+#endif
 		} else if (strcmp(name, SIMPLE_ENCODER_AMD) == 0) {
 			if (!amd_supported) {
 				changed = true;
 				name = SIMPLE_ENCODER_X264;
 				return false;
 			}
+		} else if (strcmp(name, SIMPLE_ENCODER_APPLE_H264) == 0) {
+			if (!apple_supported) {
+				changed = true;
+				name = SIMPLE_ENCODER_X264;
+				return false;
+			}
 		}
 
 		return true;
 	};
 
 	if (!CheckEncoder(curStreamEncoder))
-		config_set_string(basicConfig,
-				"SimpleOutput", "StreamEncoder",
-				curStreamEncoder);
+		config_set_string(basicConfig, "SimpleOutput", "StreamEncoder",
+				  curStreamEncoder);
 	if (!CheckEncoder(curRecEncoder))
-		config_set_string(basicConfig,
-				"SimpleOutput", "RecEncoder",
-				curRecEncoder);
+		config_set_string(basicConfig, "SimpleOutput", "RecEncoder",
+				  curRecEncoder);
 	if (changed)
 		config_save_safe(basicConfig, "tmp", nullptr);
 }
