@@ -49,12 +49,14 @@ sys.stderr = stderr_logger()\n";
 
 #if RUNTIME_LINK
 static wchar_t home_path[1024] = {0};
+static python_version_t python_version = {0};
 #endif
 
-DARRAY(char*) python_paths;
+DARRAY(char *) python_paths;
 static bool python_loaded = false;
+static bool mutexes_loaded = false;
 
-static pthread_mutex_t tick_mutex = PTHREAD_MUTEX_INITIALIZER;
+static pthread_mutex_t tick_mutex;
 static struct obs_python_script *first_tick_script = NULL;
 
 static PyObject *py_obspython = NULL;
@@ -63,21 +65,13 @@ struct python_obs_callback *cur_python_cb = NULL;
 
 /* -------------------------------------------- */
 
-bool py_to_libobs_(const char *type,
-                   PyObject *  py_in,
-                   void *      libobs_out,
-                   const char *id,
-                   const char *func,
-                   int         line)
+bool py_to_libobs_(const char *type, PyObject *py_in, void *libobs_out,
+		   const char *id, const char *func, int line)
 {
 	swig_type_info *info = SWIG_TypeQuery(type);
 	if (info == NULL) {
-		warn("%s:%d: SWIG could not find type: %s%s%s",
-		     func,
-		     line,
-		     id ? id : "",
-		     id ? "::" : "",
-		     type);
+		warn("%s:%d: SWIG could not find type: %s%s%s", func, line,
+		     id ? id : "", id ? "::" : "", type);
 		return false;
 	}
 
@@ -85,33 +79,21 @@ bool py_to_libobs_(const char *type,
 	if (!SWIG_IsOK(ret)) {
 		warn("%s:%d: SWIG failed to convert python object to obs "
 		     "object: %s%s%s",
-		     func,
-		     line,
-		     id ? id : "",
-		     id ? "::" : "",
-		     type);
+		     func, line, id ? id : "", id ? "::" : "", type);
 		return false;
 	}
 
 	return true;
 }
 
-bool libobs_to_py_(const char *type,
-                   void *      libobs_in,
-                   bool        ownership,
-                   PyObject ** py_out,
-                   const char *id,
-                   const char *func,
-                   int         line)
+bool libobs_to_py_(const char *type, void *libobs_in, bool ownership,
+		   PyObject **py_out, const char *id, const char *func,
+		   int line)
 {
 	swig_type_info *info = SWIG_TypeQuery(type);
 	if (info == NULL) {
-		warn("%s:%d: SWIG could not find type: %s%s%s",
-		     func,
-		     line,
-		     id ? id : "",
-		     id ? "::" : "",
-		     type);
+		warn("%s:%d: SWIG could not find type: %s%s%s", func, line,
+		     id ? id : "", id ? "::" : "", type);
 		return false;
 	}
 
@@ -119,33 +101,27 @@ bool libobs_to_py_(const char *type,
 	if (*py_out == Py_None) {
 		warn("%s:%d: SWIG failed to convert obs object to python "
 		     "object: %s%s%s",
-		     func,
-		     line,
-		     id ? id : "",
-		     id ? "::" : "",
-		     type);
+		     func, line, id ? id : "", id ? "::" : "", type);
 		return false;
 	}
 
 	return true;
 }
 
-
-#define libobs_to_py(type, obs_obj, ownership, py_obj) \
-	libobs_to_py_(#type " *", obs_obj, ownership, py_obj, \
-			NULL, __func__, __LINE__)
+#define libobs_to_py(type, obs_obj, ownership, py_obj)                        \
+	libobs_to_py_(#type " *", obs_obj, ownership, py_obj, NULL, __func__, \
+		      __LINE__)
 #define py_to_libobs(type, py_obj, libobs_out) \
-	py_to_libobs_(#type " *", py_obj, libobs_out, \
-			NULL, __func__, __LINE__)
+	py_to_libobs_(#type " *", py_obj, libobs_out, NULL, __func__, __LINE__)
 
-#define lock_callback(cb) \
-	lock_python(); \
-	struct obs_python_script *__last_script = cur_python_script; \
-	struct python_obs_callback *__last_cb = cur_python_cb; \
+#define lock_callback(cb)                                                \
+	lock_python();                                                   \
+	struct obs_python_script *__last_script = cur_python_script;     \
+	struct python_obs_callback *__last_cb = cur_python_cb;           \
 	cur_python_script = (struct obs_python_script *)cb->base.script; \
 	cur_python_cb = cb
-#define unlock_callback() \
-	cur_python_cb = __last_cb; \
+#define unlock_callback()                  \
+	cur_python_cb = __last_cb;         \
 	cur_python_script = __last_script; \
 	unlock_python()
 
@@ -173,9 +149,13 @@ void add_functions_to_py_module(PyObject *module, PyMethodDef *method_list)
 
 static PyObject *py_get_current_script_path(PyObject *self, PyObject *args)
 {
+	PyObject *dir;
+
 	UNUSED_PARAMETER(args);
-	return PyDict_GetItemString(PyModule_GetDict(self),
-			"__script_dir__");
+
+	dir = PyDict_GetItemString(PyModule_GetDict(self), "__script_dir__");
+	Py_XINCREF(dir);
+	return dir;
 }
 
 static void get_defaults(struct obs_python_script *data, PyObject *get_defs)
@@ -194,19 +174,19 @@ static void get_defaults(struct obs_python_script *data, PyObject *get_defs)
 
 static bool load_python_script(struct obs_python_script *data)
 {
-	PyObject *py_file     = NULL;
-	PyObject *py_module   = NULL;
-	PyObject *py_success  = NULL;
-	PyObject *py_tick     = NULL;
-	PyObject *py_load     = NULL;
+	PyObject *py_file = NULL;
+	PyObject *py_module = NULL;
+	PyObject *py_success = NULL;
+	PyObject *py_tick = NULL;
+	PyObject *py_load = NULL;
 	PyObject *py_defaults = NULL;
-	bool      success     = false;
-	int       ret;
+	bool success = false;
+	int ret;
 
 	cur_python_script = data;
 
 	if (!data->module) {
-		py_file   = PyUnicode_FromString(data->name.array);
+		py_file = PyUnicode_FromString(data->name.array);
 		py_module = PyImport_Import(py_file);
 	} else {
 		py_module = PyImport_ReloadModule(data->module);
@@ -223,7 +203,7 @@ static bool load_python_script(struct obs_python_script *data)
 		goto fail;
 
 	ret = PyModule_AddStringConstant(py_module, "__script_dir__",
-			data->dir.array);
+					 data->dir.array);
 	if (py_error() || ret != 0)
 		goto fail;
 
@@ -233,12 +213,9 @@ static bool load_python_script(struct obs_python_script *data)
 		goto fail;
 
 	static PyMethodDef global_funcs[] = {
-		{"script_path",
-		 py_get_current_script_path,
-		 METH_NOARGS,
+		{"script_path", py_get_current_script_path, METH_NOARGS,
 		 "Gets the script path"},
-		{0}
-	};
+		{0}};
 
 	add_functions_to_py_module(py_module, global_funcs);
 
@@ -250,8 +227,8 @@ static bool load_python_script(struct obs_python_script *data)
 	if (!data->save)
 		PyErr_Clear();
 
-	data->get_properties = PyObject_GetAttrString(py_module,
-			"script_properties");
+	data->get_properties =
+		PyObject_GetAttrString(py_module, "script_properties");
 	if (!data->get_properties)
 		PyErr_Clear();
 
@@ -270,7 +247,8 @@ static bool load_python_script(struct obs_python_script *data)
 		PyObject *py_desc = PyUnicode_AsUTF8String(py_ret);
 		if (py_desc) {
 			const char *desc = PyBytes_AS_STRING(py_desc);
-			if (desc) dstr_copy(&data->base.desc, desc);
+			if (desc)
+				dstr_copy(&data->base.desc, desc);
 			Py_DECREF(py_desc);
 		}
 		Py_XDECREF(py_ret);
@@ -286,7 +264,8 @@ static bool load_python_script(struct obs_python_script *data)
 		struct obs_python_script *next = first_tick_script;
 		data->next_tick = next;
 		data->p_prev_next_tick = &first_tick_script;
-		if (next) next->p_prev_next_tick = &data->next_tick;
+		if (next)
+			next->p_prev_next_tick = &data->next_tick;
 		first_tick_script = data;
 
 		data->tick = py_tick;
@@ -332,9 +311,9 @@ fail:
 
 static void unload_python_script(struct obs_python_script *data)
 {
-	PyObject *py_module   = data->module;
-	PyObject *py_func     = NULL;
-	PyObject *py_ret      = NULL;
+	PyObject *py_module = data->module;
+	PyObject *py_func = NULL;
+	PyObject *py_ret = NULL;
 
 	cur_python_script = data;
 
@@ -358,8 +337,8 @@ fail:
 static void add_to_python_path(const char *path)
 {
 	PyObject *py_path_str = NULL;
-	PyObject *py_path     = NULL;
-	int       ret;
+	PyObject *py_path = NULL;
+	int ret;
 
 	if (!path || !*path)
 		return;
@@ -401,7 +380,7 @@ struct python_obs_timer {
 	uint64_t interval;
 };
 
-static pthread_mutex_t timer_mutex = PTHREAD_MUTEX_INITIALIZER;
+static pthread_mutex_t timer_mutex;
 static struct python_obs_timer *first_timer = NULL;
 
 static inline void python_obs_timer_init(struct python_obs_timer *timer)
@@ -411,7 +390,8 @@ static inline void python_obs_timer_init(struct python_obs_timer *timer)
 	struct python_obs_timer *next = first_timer;
 	timer->next = next;
 	timer->p_prev_next = &first_timer;
-	if (next) next->p_prev_next = &timer->next;
+	if (next)
+		next->p_prev_next = &timer->next;
 	first_timer = timer;
 
 	pthread_mutex_unlock(&timer_mutex);
@@ -420,12 +400,13 @@ static inline void python_obs_timer_init(struct python_obs_timer *timer)
 static inline void python_obs_timer_remove(struct python_obs_timer *timer)
 {
 	struct python_obs_timer *next = timer->next;
-	if (next) next->p_prev_next = timer->p_prev_next;
+	if (next)
+		next->p_prev_next = timer->p_prev_next;
 	*timer->p_prev_next = timer->next;
 }
 
-static inline struct python_obs_callback *python_obs_timer_cb(
-		struct python_obs_timer *timer)
+static inline struct python_obs_callback *
+python_obs_timer_cb(struct python_obs_timer *timer)
 {
 	return &((struct python_obs_callback *)timer)[-1];
 }
@@ -440,8 +421,10 @@ static PyObject *timer_remove(PyObject *self, PyObject *args)
 	if (!parse_args(args, "O", &py_cb))
 		return python_none();
 
-	struct python_obs_callback *cb = find_python_obs_callback(script, py_cb);
-	if (cb) remove_python_obs_callback(cb);
+	struct python_obs_callback *cb =
+		find_python_obs_callback(script, py_cb);
+	if (cb)
+		remove_python_obs_callback(cb);
 	return python_none();
 }
 
@@ -449,7 +432,7 @@ static void timer_call(struct script_callback *p_cb)
 {
 	struct python_obs_callback *cb = (struct python_obs_callback *)p_cb;
 
-	if (p_cb->removed)
+	if (script_callback_removed(p_cb))
 		return;
 
 	lock_callback(cb);
@@ -478,7 +461,7 @@ static PyObject *timer_add(PyObject *self, PyObject *args)
 		return python_none();
 
 	struct python_obs_callback *cb = add_python_obs_callback_extra(
-			script, py_cb, sizeof(struct python_obs_timer));
+		script, py_cb, sizeof(struct python_obs_timer));
 	struct python_obs_timer *timer = python_obs_callback_extra_data(cb);
 
 	timer->interval = (uint64_t)ms * 1000000ULL;
@@ -494,7 +477,7 @@ static void obs_python_tick_callback(void *priv, float seconds)
 {
 	struct python_obs_callback *cb = priv;
 
-	if (cb->base.removed) {
+	if (script_callback_removed(&cb->base)) {
 		obs_remove_tick_callback(obs_python_tick_callback, cb);
 		return;
 	}
@@ -528,8 +511,10 @@ static PyObject *obs_python_remove_tick_callback(PyObject *self, PyObject *args)
 	if (!py_cb || !PyFunction_Check(py_cb))
 		return python_none();
 
-	struct python_obs_callback *cb = find_python_obs_callback(script, py_cb);
-	if (cb) remove_python_obs_callback(cb);
+	struct python_obs_callback *cb =
+		find_python_obs_callback(script, py_cb);
+	if (cb)
+		remove_python_obs_callback(cb);
 	return python_none();
 }
 
@@ -562,7 +547,7 @@ static void calldata_signal_callback(void *priv, calldata_t *cd)
 {
 	struct python_obs_callback *cb = priv;
 
-	if (cb->base.removed) {
+	if (script_callback_removed(&cb->base)) {
 		signal_handler_remove_current();
 		return;
 	}
@@ -583,8 +568,8 @@ static void calldata_signal_callback(void *priv, calldata_t *cd)
 	unlock_callback();
 }
 
-static PyObject *obs_python_signal_handler_disconnect(
-		PyObject *self, PyObject *args)
+static PyObject *obs_python_signal_handler_disconnect(PyObject *self,
+						      PyObject *args)
 {
 	struct obs_python_script *script = cur_python_script;
 	PyObject *py_sh = NULL;
@@ -609,27 +594,28 @@ static PyObject *obs_python_signal_handler_disconnect(
 	if (!py_cb || !PyFunction_Check(py_cb))
 		return python_none();
 
-	struct python_obs_callback *cb = find_python_obs_callback(script, py_cb);
+	struct python_obs_callback *cb =
+		find_python_obs_callback(script, py_cb);
 	while (cb) {
 		signal_handler_t *cb_handler =
 			calldata_ptr(&cb->base.extra, "handler");
 		const char *cb_signal =
 			calldata_string(&cb->base.extra, "signal");
 
-		if (cb_signal &&
-		    strcmp(signal, cb_signal) != 0 &&
+		if (cb_signal && strcmp(signal, cb_signal) == 0 &&
 		    handler == cb_handler)
 			break;
 
 		cb = find_next_python_obs_callback(script, cb, py_cb);
 	}
 
-	if (cb) remove_python_obs_callback(cb);
+	if (cb)
+		remove_python_obs_callback(cb);
 	return python_none();
 }
 
-static PyObject *obs_python_signal_handler_connect(
-		PyObject *self, PyObject *args)
+static PyObject *obs_python_signal_handler_connect(PyObject *self,
+						   PyObject *args)
 {
 	struct obs_python_script *script = cur_python_script;
 	PyObject *py_sh = NULL;
@@ -663,11 +649,11 @@ static PyObject *obs_python_signal_handler_connect(
 /* -------------------------------------------- */
 
 static void calldata_signal_callback_global(void *priv, const char *signal,
-		calldata_t *cd)
+					    calldata_t *cd)
 {
 	struct python_obs_callback *cb = priv;
 
-	if (cb->base.removed) {
+	if (script_callback_removed(&cb->base)) {
 		signal_handler_remove_current();
 		return;
 	}
@@ -688,8 +674,8 @@ static void calldata_signal_callback_global(void *priv, const char *signal,
 	unlock_callback();
 }
 
-static PyObject *obs_python_signal_handler_disconnect_global(
-		PyObject *self, PyObject *args)
+static PyObject *obs_python_signal_handler_disconnect_global(PyObject *self,
+							     PyObject *args)
 {
 	struct obs_python_script *script = cur_python_script;
 	PyObject *py_sh = NULL;
@@ -713,7 +699,8 @@ static PyObject *obs_python_signal_handler_disconnect_global(
 	if (!py_cb || !PyFunction_Check(py_cb))
 		return python_none();
 
-	struct python_obs_callback *cb = find_python_obs_callback(script, py_cb);
+	struct python_obs_callback *cb =
+		find_python_obs_callback(script, py_cb);
 	while (cb) {
 		signal_handler_t *cb_handler =
 			calldata_ptr(&cb->base.extra, "handler");
@@ -724,12 +711,13 @@ static PyObject *obs_python_signal_handler_disconnect_global(
 		cb = find_next_python_obs_callback(script, cb, py_cb);
 	}
 
-	if (cb) remove_python_obs_callback(cb);
+	if (cb)
+		remove_python_obs_callback(cb);
 	return python_none();
 }
 
-static PyObject *obs_python_signal_handler_connect_global(
-		PyObject *self, PyObject *args)
+static PyObject *obs_python_signal_handler_connect_global(PyObject *self,
+							  PyObject *args)
 {
 	struct obs_python_script *script = cur_python_script;
 	PyObject *py_sh = NULL;
@@ -755,8 +743,8 @@ static PyObject *obs_python_signal_handler_connect_global(
 
 	struct python_obs_callback *cb = add_python_obs_callback(script, py_cb);
 	calldata_set_ptr(&cb->base.extra, "handler", handler);
-	signal_handler_connect_global(handler,
-			calldata_signal_callback_global, cb);
+	signal_handler_connect_global(handler, calldata_signal_callback_global,
+				      cb);
 	return python_none();
 }
 
@@ -773,14 +761,14 @@ static void on_remove_hotkey(void *p_cb)
 	obs_hotkey_id id = (obs_hotkey_id)calldata_int(&cb->base.extra, "id");
 
 	if (id != OBS_INVALID_HOTKEY_ID)
-		defer_call_post(defer_hotkey_unregister, (void*)(uintptr_t)id);
+		defer_call_post(defer_hotkey_unregister, (void *)(uintptr_t)id);
 }
 
 static void hotkey_pressed(void *p_cb, bool pressed)
 {
 	struct python_obs_callback *cb = p_cb;
 
-	if (cb->base.removed)
+	if (script_callback_removed(&cb->base))
 		return;
 
 	lock_callback(cb);
@@ -811,12 +799,12 @@ static inline PyObject *py_invalid_hotkey_id()
 	return PyLong_FromUnsignedLongLong(OBS_INVALID_HOTKEY_ID);
 }
 
-static void hotkey_callback(void *p_cb, obs_hotkey_id id,
-		obs_hotkey_t *hotkey, bool pressed)
+static void hotkey_callback(void *p_cb, obs_hotkey_id id, obs_hotkey_t *hotkey,
+			    bool pressed)
 {
 	struct python_obs_callback *cb = p_cb;
 
-	if (cb->base.removed)
+	if (script_callback_removed(&cb->base))
 		return;
 
 	if (pressed)
@@ -844,8 +832,10 @@ static PyObject *hotkey_unregister(PyObject *self, PyObject *args)
 	if (!py_cb || !PyFunction_Check(py_cb))
 		return python_none();
 
-	struct python_obs_callback *cb = find_python_obs_callback(script, py_cb);
-	if (cb) remove_python_obs_callback(cb);
+	struct python_obs_callback *cb =
+		find_python_obs_callback(script, py_cb);
+	if (cb)
+		remove_python_obs_callback(cb);
 
 	UNUSED_PARAMETER(self);
 	return python_none();
@@ -879,12 +869,12 @@ static PyObject *hotkey_register_frontend(PyObject *self, PyObject *args)
 /* -------------------------------------------- */
 
 static bool button_prop_clicked(obs_properties_t *props, obs_property_t *p,
-		void *p_cb)
+				void *p_cb)
 {
 	struct python_obs_callback *cb = p_cb;
 	bool ret = false;
 
-	if (cb->base.removed)
+	if (script_callback_removed(&cb->base))
 		return false;
 
 	lock_callback(cb);
@@ -931,7 +921,7 @@ static PyObject *properties_add_button(PyObject *self, PyObject *args)
 
 	struct python_obs_callback *cb = add_python_obs_callback(script, py_cb);
 	p = obs_properties_add_button2(props, name, text, button_prop_clicked,
-			cb);
+				       cb);
 
 	if (!p || !libobs_to_py(obs_property_t, p, false, &py_ret))
 		return python_none();
@@ -943,12 +933,12 @@ static PyObject *properties_add_button(PyObject *self, PyObject *args)
 /* -------------------------------------------- */
 
 static bool modified_callback(void *p_cb, obs_properties_t *props,
-		obs_property_t *p, obs_data_t *settings)
+			      obs_property_t *p, obs_data_t *settings)
 {
 	struct python_obs_callback *cb = p_cb;
 	bool ret = false;
 
-	if (cb->base.removed)
+	if (script_callback_removed(&cb->base))
 		return false;
 
 	lock_callback(cb);
@@ -961,8 +951,8 @@ static bool modified_callback(void *p_cb, obs_properties_t *props,
 	    libobs_to_py(obs_property_t, p, false, &py_p) &&
 	    libobs_to_py(obs_data_t, settings, false, &py_settings)) {
 
-		PyObject *args = Py_BuildValue("(OOO)", py_props, py_p,
-				py_settings);
+		PyObject *args =
+			Py_BuildValue("(OOO)", py_props, py_p, py_settings);
 		PyObject *py_ret = PyObject_CallObject(cb->func, args);
 		if (!py_error())
 			ret = py_ret == Py_True;
@@ -1017,7 +1007,7 @@ static PyObject *remove_current_callback(PyObject *self, PyObject *args)
 static PyObject *calldata_source(PyObject *self, PyObject *args)
 {
 	PyObject *py_ret = NULL;
-	PyObject *py_cd  = NULL;
+	PyObject *py_cd = NULL;
 
 	calldata_t *cd;
 	const char *name;
@@ -1039,7 +1029,7 @@ fail:
 static PyObject *calldata_sceneitem(PyObject *self, PyObject *args)
 {
 	PyObject *py_ret = NULL;
-	PyObject *py_cd  = NULL;
+	PyObject *py_cd = NULL;
 
 	calldata_t *cd;
 	const char *name;
@@ -1086,7 +1076,7 @@ static PyObject *enum_sources(PyObject *self, PyObject *args)
 /* -------------------------------------------- */
 
 static bool enum_items_proc(obs_scene_t *scene, obs_sceneitem_t *item,
-		void *param)
+			    void *param)
 {
 	PyObject *list = param;
 	PyObject *py_item;
@@ -1118,6 +1108,23 @@ static PyObject *scene_enum_items(PyObject *self, PyObject *args)
 	return list;
 }
 
+static PyObject *sceneitem_group_enum_items(PyObject *self, PyObject *args)
+{
+	PyObject *py_sceneitem;
+	obs_sceneitem_t *sceneitem;
+
+	UNUSED_PARAMETER(self);
+
+	if (!parse_args(args, "O", &py_sceneitem))
+		return python_none();
+	if (!py_to_libobs(obs_sceneitem_t, py_sceneitem, &sceneitem))
+		return python_none();
+
+	PyObject *list = PyList_New(0);
+	obs_sceneitem_group_enum_items(sceneitem, enum_items_proc, list);
+	return list;
+}
+
 /* -------------------------------------------- */
 
 static PyObject *source_list_release(PyObject *self, PyObject *args)
@@ -1165,7 +1172,7 @@ static PyObject *sceneitem_list_release(PyObject *self, PyObject *args)
 struct dstr cur_py_log_chunk = {0};
 
 static PyObject *py_script_log_internal(PyObject *self, PyObject *args,
-		bool add_endl)
+					bool add_endl)
 {
 	static bool calling_self = false;
 	int log_level;
@@ -1195,7 +1202,7 @@ static PyObject *py_script_log_internal(PyObject *self, PyObject *args,
 		*endl = 0;
 		if (cur_python_script)
 			script_log(&cur_python_script->base, log_level, "%s",
-					start);
+				   start);
 		else
 			script_log(NULL, log_level, "%s", start);
 		*endl = '\n';
@@ -1206,7 +1213,8 @@ static PyObject *py_script_log_internal(PyObject *self, PyObject *args,
 
 	if (start) {
 		size_t len = strlen(start);
-		if (len) memmove(cur_py_log_chunk.array, start, len);
+		if (len)
+			memmove(cur_py_log_chunk.array, start, len);
 		dstr_resize(&cur_py_log_chunk, len);
 	}
 
@@ -1244,32 +1252,29 @@ static void add_hook_functions(PyObject *module)
 		DEF_FUNC("sceneitem_list_release", sceneitem_list_release),
 		DEF_FUNC("obs_enum_sources", enum_sources),
 		DEF_FUNC("obs_scene_enum_items", scene_enum_items),
+		DEF_FUNC("obs_sceneitem_group_enum_items",
+			 sceneitem_group_enum_items),
 		DEF_FUNC("obs_remove_tick_callback",
-		         obs_python_remove_tick_callback),
-		DEF_FUNC("obs_add_tick_callback",
-		         obs_python_add_tick_callback),
+			 obs_python_remove_tick_callback),
+		DEF_FUNC("obs_add_tick_callback", obs_python_add_tick_callback),
 		DEF_FUNC("signal_handler_disconnect",
-		         obs_python_signal_handler_disconnect),
+			 obs_python_signal_handler_disconnect),
 		DEF_FUNC("signal_handler_connect",
-		         obs_python_signal_handler_connect),
+			 obs_python_signal_handler_connect),
 		DEF_FUNC("signal_handler_disconnect_global",
-		         obs_python_signal_handler_disconnect_global),
+			 obs_python_signal_handler_disconnect_global),
 		DEF_FUNC("signal_handler_connect_global",
-		         obs_python_signal_handler_connect_global),
-		DEF_FUNC("obs_hotkey_unregister",
-			hotkey_unregister),
+			 obs_python_signal_handler_connect_global),
+		DEF_FUNC("obs_hotkey_unregister", hotkey_unregister),
 		DEF_FUNC("obs_hotkey_register_frontend",
-			hotkey_register_frontend),
-		DEF_FUNC("obs_properties_add_button",
-			properties_add_button),
+			 hotkey_register_frontend),
+		DEF_FUNC("obs_properties_add_button", properties_add_button),
 		DEF_FUNC("obs_property_set_modified_callback",
-			property_set_modified_callback),
-		DEF_FUNC("remove_current_callback",
-		         remove_current_callback),
+			 property_set_modified_callback),
+		DEF_FUNC("remove_current_callback", remove_current_callback),
 
 #undef DEF_FUNC
-		{0}
-	};
+		{0}};
 
 	add_functions_to_py_module(module, funcs);
 }
@@ -1288,8 +1293,12 @@ bool obs_python_script_load(obs_script_t *s)
 		data->base.loaded = load_python_script(data);
 		unlock_python();
 
-		if (data->base.loaded)
+		if (data->base.loaded) {
+			blog(LOG_INFO,
+			     "[obs-scripting]: Loaded python script: %s",
+			     data->base.file.array);
 			obs_python_script_update(s, NULL);
+		}
 	}
 
 	return data->base.loaded;
@@ -1332,6 +1341,8 @@ obs_script_t *obs_python_script_create(const char *path, obs_data_t *settings)
 	add_to_python_path(data->dir.array);
 	data->base.loaded = load_python_script(data);
 	if (data->base.loaded) {
+		blog(LOG_INFO, "[obs-scripting]: Loaded python script: %s",
+		     data->base.file.array);
 		cur_python_script = data;
 		obs_python_script_update(&data->base, NULL);
 		cur_python_script = NULL;
@@ -1348,6 +1359,24 @@ void obs_python_script_unload(obs_script_t *s)
 	if (!s->loaded || !python_loaded)
 		return;
 
+	/* ---------------------------- */
+	/* mark callbacks as removed    */
+
+	lock_python();
+
+	/* XXX: scripts can potentially make callbacks when this happens, so
+	 * this probably still isn't ideal as we can't predict how the
+	 * processor or operating system is going to schedule things. a more
+	 * ideal method would be to reference count the script objects and
+	 * atomically share ownership with callbacks when they're called. */
+	struct script_callback *cb = data->first_callback;
+	while (cb) {
+		os_atomic_set_bool(&cb->removed, true);
+		cb = cb->next;
+	}
+
+	unlock_python();
+
 	/* ---------------------------- */
 	/* unhook tick function         */
 
@@ -1355,7 +1384,8 @@ void obs_python_script_unload(obs_script_t *s)
 		pthread_mutex_lock(&tick_mutex);
 
 		struct obs_python_script *next = data->next_tick;
-		if (next) next->p_prev_next_tick = data->p_prev_next_tick;
+		if (next)
+			next->p_prev_next_tick = data->p_prev_next_tick;
 		*data->p_prev_next_tick = next;
 
 		pthread_mutex_unlock(&tick_mutex);
@@ -1364,7 +1394,7 @@ void obs_python_script_unload(obs_script_t *s)
 		data->next_tick = NULL;
 	}
 
-	lock_python();
+	relock_python();
 
 	Py_XDECREF(data->tick);
 	Py_XDECREF(data->save);
@@ -1378,7 +1408,7 @@ void obs_python_script_unload(obs_script_t *s)
 	/* ---------------------------- */
 	/* remove all callbacks         */
 
-	struct script_callback *cb = data->first_callback;
+	cb = data->first_callback;
 	while (cb) {
 		struct script_callback *next = cb->next;
 		remove_script_callback(cb);
@@ -1392,6 +1422,9 @@ void obs_python_script_unload(obs_script_t *s)
 	unlock_python();
 
 	s->loaded = false;
+
+	blog(LOG_INFO, "[obs-scripting]: Unloaded python script: %s",
+	     data->base.file.array);
 }
 
 void obs_python_script_destroy(obs_script_t *s)
@@ -1500,6 +1533,12 @@ void obs_python_script_save(obs_script_t *s)
 static void python_tick(void *param, float seconds)
 {
 	struct obs_python_script *data;
+	/* When loading a new Python script, the GIL might be released while
+	 * importing the module, allowing the tick to run and change and reset
+	 * the cur_python_script state variable. Use the busy_script variable
+	 * to save and restore the value if not null.
+	 */
+	struct obs_python_script *busy_script;
 	bool valid;
 	uint64_t ts = obs_get_video_frame_time();
 
@@ -1517,10 +1556,15 @@ static void python_tick(void *param, float seconds)
 
 		pthread_mutex_lock(&tick_mutex);
 		data = first_tick_script;
+
+		if (cur_python_script)
+			busy_script = cur_python_script;
+
 		while (data) {
 			cur_python_script = data;
 
-			PyObject *py_ret = PyObject_CallObject(data->tick, args);
+			PyObject *py_ret =
+				PyObject_CallObject(data->tick, args);
 			Py_XDECREF(py_ret);
 			py_error();
 
@@ -1528,6 +1572,10 @@ static void python_tick(void *param, float seconds)
 		}
 
 		cur_python_script = NULL;
+		if (busy_script) {
+			cur_python_script = busy_script;
+			busy_script = NULL;
+		}
 
 		pthread_mutex_unlock(&tick_mutex);
 
@@ -1545,7 +1593,7 @@ static void python_tick(void *param, float seconds)
 		struct python_obs_timer *next = timer->next;
 		struct python_obs_callback *cb = python_obs_timer_cb(timer);
 
-		if (cb->base.removed) {
+		if (script_callback_removed(&cb->base)) {
 			python_obs_timer_remove(timer);
 		} else {
 			uint64_t elapsed = ts - timer->last_ts;
@@ -1575,6 +1623,17 @@ bool obs_scripting_python_runtime_linked(void)
 	return (bool)RUNTIME_LINK;
 }
 
+void obs_scripting_python_version(char *version, size_t version_length)
+{
+#if RUNTIME_LINK
+	snprintf(version, version_length, "%d.%d", python_version.major,
+		 python_version.minor);
+#else
+	snprintf(version, version_length, "%d.%d", PY_MAJOR_VERSION,
+		 PY_MINOR_VERSION);
+#endif
+}
+
 bool obs_scripting_python_loaded(void)
 {
 	return python_loaded;
@@ -1584,40 +1643,38 @@ void obs_python_load(void)
 {
 	da_init(python_paths);
 
-	pthread_mutexattr_t attr;
-	pthread_mutexattr_init(&attr);
-	pthread_mutexattr_settype(&attr, PTHREAD_MUTEX_RECURSIVE);
-
 	pthread_mutex_init(&tick_mutex, NULL);
-	pthread_mutex_init(&timer_mutex, &attr);
+	pthread_mutex_init_recursive(&timer_mutex);
 
+	mutexes_loaded = true;
 }
 
 extern void add_python_frontend_funcs(PyObject *module);
 
+static bool python_loaded_at_all = false;
+
 bool obs_scripting_load_python(const char *python_path)
 {
 	if (python_loaded)
 		return true;
 
-	/* Use external python on windows and mac */
+		/* Use external python on windows and mac */
 #if RUNTIME_LINK
-# if 0
-	struct dstr old_path  = {0};
-	struct dstr new_path  = {0};
-# endif
-
-	if (!import_python(python_path))
+	if (!import_python(python_path, &python_version))
 		return false;
 
 	if (python_path && *python_path) {
+#ifdef __APPLE__
+		char temp[PATH_MAX];
+		sprintf(temp, "%s/Python.framework/Versions/Current",
+			python_path);
+		os_utf8_to_wcs(temp, 0, home_path, PATH_MAX);
+		Py_SetPythonHome(home_path);
+#else
+
 		os_utf8_to_wcs(python_path, 0, home_path, 1024);
 		Py_SetPythonHome(home_path);
-# if 0
-		dstr_copy(&old_path, getenv("PATH"));
-		_putenv("PYTHONPATH=");
-		_putenv("PATH=");
-# endif
+#endif
 	}
 #else
 	UNUSED_PARAMETER(python_path);
@@ -1627,28 +1684,23 @@ bool obs_scripting_load_python(const char *python_path)
 	if (!Py_IsInitialized())
 		return false;
 
-#if 0
-# ifdef _DEBUG
-	if (pythondir && *pythondir) {
-		dstr_printf(&new_path, "PATH=%s", old_path.array);
-		_putenv(new_path.array);
-	}
-# endif
-
-	bfree(pythondir);
-	dstr_free(&new_path);
-	dstr_free(&old_path);
+#if RUNTIME_LINK
+	if (python_version.major == 3 && python_version.minor < 7) {
+#elif PY_VERSION_HEX < 0x030700b0
+	if (true) {
+#else
+	if (false) {
 #endif
-
-	PyEval_InitThreads();
-	if (!PyEval_ThreadsInitialized())
-		return false;
+		PyEval_InitThreads();
+		if (!PyEval_ThreadsInitialized())
+			return false;
+	}
 
 	/* ---------------------------------------------- */
 	/* Must set arguments for guis to work            */
 
 	wchar_t *argv[] = {L"", NULL};
-	int      argc   = sizeof(argv) / sizeof(wchar_t*) - 1;
+	int argc = sizeof(argv) / sizeof(wchar_t *) - 1;
 
 	PySys_SetArgv(argc, argv);
 
@@ -1667,7 +1719,34 @@ bool obs_scripting_load_python(const char *python_path)
 	/* ---------------------------------------------- */
 	/* Load main interface module                     */
 
-	add_to_python_path(SCRIPT_DIR);
+#ifdef __APPLE__
+	struct dstr plugin_path;
+	struct dstr resource_path;
+
+	dstr_init_move_array(&plugin_path, os_get_executable_path_ptr(""));
+	dstr_init_copy(&resource_path, plugin_path.array);
+	dstr_cat(&plugin_path, "../PlugIns");
+	dstr_cat(&resource_path, "../Resources");
+
+	char *absolute_plugin_path = os_get_abs_path_ptr(plugin_path.array);
+	char *absolute_resource_path = os_get_abs_path_ptr(resource_path.array);
+
+	if (absolute_plugin_path != NULL) {
+		add_to_python_path(absolute_plugin_path);
+		bfree(absolute_plugin_path);
+	}
+	dstr_free(&plugin_path);
+
+	if (absolute_resource_path != NULL) {
+		add_to_python_path(absolute_resource_path);
+		bfree(absolute_resource_path);
+	}
+	dstr_free(&resource_path);
+#else
+	char *absolute_script_path = os_get_abs_path_ptr(SCRIPT_DIR);
+	add_to_python_path(absolute_script_path);
+	bfree(absolute_script_path);
+#endif
 
 	py_obspython = PyImport_ImportModule("obspython");
 	bool success = !py_error();
@@ -1696,6 +1775,8 @@ out:
 		obs_python_unload();
 	}
 
+	python_loaded_at_all = success;
+
 	if (python_loaded)
 		obs_add_tick_callback(python_tick, NULL);
 
@@ -1704,6 +1785,14 @@ out:
 
 void obs_python_unload(void)
 {
+	if (mutexes_loaded) {
+		pthread_mutex_destroy(&tick_mutex);
+		pthread_mutex_destroy(&timer_mutex);
+	}
+
+	if (!python_loaded_at_all)
+		return;
+
 	if (python_loaded && Py_IsInitialized()) {
 		PyGILState_Ensure();
 
@@ -1719,7 +1808,7 @@ void obs_python_unload(void)
 		bfree(python_paths.array[i]);
 	da_free(python_paths);
 
-	pthread_mutex_destroy(&tick_mutex);
-	pthread_mutex_destroy(&timer_mutex);
 	dstr_free(&cur_py_log_chunk);
+
+	python_loaded_at_all = false;
 }
