@@ -14,6 +14,14 @@
  * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
+#include "obsconfig.h"
+
+#if !defined(__APPLE__) && OBS_QT_VERSION == 6
+#define _GNU_SOURCE
+#include <link.h>
+#include <stdlib.h>
+#endif
+
 #include <stdio.h>
 #include <errno.h>
 #include <sys/types.h>
@@ -28,18 +36,20 @@
 #include <time.h>
 #include <signal.h>
 
-#include "obsconfig.h"
-
 #if !defined(__APPLE__)
 #include <sys/times.h>
 #include <sys/wait.h>
-#ifdef __FreeBSD__
+#include <libgen.h>
+#if defined(__FreeBSD__) || defined(__OpenBSD__)
 #include <sys/param.h>
 #include <sys/queue.h>
 #include <sys/socket.h>
+#include <sys/sysctl.h>
 #include <sys/user.h>
 #include <unistd.h>
+#if defined(__FreeBSD__)
 #include <libprocstat.h>
+#endif
 #else
 #include <sys/resource.h>
 #endif
@@ -60,16 +70,22 @@ void *os_dlopen(const char *path)
 
 	dstr_init_copy(&dylib_name, path);
 #ifdef __APPLE__
-	if (!dstr_find(&dylib_name, ".so") && !dstr_find(&dylib_name, ".dylib"))
+	if (!dstr_find(&dylib_name, ".framework") &&
+	    !dstr_find(&dylib_name, ".plugin") &&
+	    !dstr_find(&dylib_name, ".dylib") && !dstr_find(&dylib_name, ".so"))
 #else
 	if (!dstr_find(&dylib_name, ".so"))
 #endif
 		dstr_cat(&dylib_name, ".so");
 
+#ifdef __APPLE__
+	void *res = dlopen(dylib_name.array, RTLD_LAZY | RTLD_FIRST);
+#else
 	void *res = dlopen(dylib_name.array, RTLD_LAZY);
+#endif
 	if (!res)
-		blog(LOG_ERROR, "os_dlopen(%s->%s): %s\n",
-				path, dylib_name.array, dlerror());
+		blog(LOG_ERROR, "os_dlopen(%s->%s): %s\n", path,
+		     dylib_name.array, dlerror());
 
 	dstr_free(&dylib_name);
 	return res;
@@ -86,6 +102,61 @@ void os_dlclose(void *module)
 		dlclose(module);
 }
 
+#if !defined(__APPLE__) && OBS_QT_VERSION == 6
+int module_has_qt5_check(const char *path)
+{
+	void *mod = os_dlopen(path);
+	if (mod == NULL) {
+		return 1;
+	}
+
+	struct link_map *list = NULL;
+	if (dlinfo(mod, RTLD_DI_LINKMAP, &list) == 0) {
+		for (struct link_map *ptr = list; ptr; ptr = ptr->l_next) {
+			if (strstr(ptr->l_name, "libQt5") != NULL) {
+				return 0;
+			}
+		}
+	}
+
+	return 1;
+}
+
+bool has_qt5_dependency(const char *path)
+{
+	pid_t pid = fork();
+	if (pid == 0) {
+		_exit(module_has_qt5_check(path));
+	}
+	if (pid < 0) {
+		return false;
+	}
+	int status;
+	if (waitpid(pid, &status, 0) < 0) {
+		return false;
+	}
+	return WIFEXITED(status) && WEXITSTATUS(status) == 0;
+}
+#endif
+
+void get_plugin_info(const char *path, bool *is_obs_plugin, bool *can_load)
+{
+	*is_obs_plugin = true;
+	*can_load = true;
+#if !defined(__APPLE__) && OBS_QT_VERSION == 6
+	*can_load = !has_qt5_dependency(path);
+#endif
+	UNUSED_PARAMETER(path);
+}
+
+bool os_is_obs_plugin(const char *path)
+{
+	UNUSED_PARAMETER(path);
+
+	/* not necessary on this platform */
+	return true;
+}
+
 #if !defined(__APPLE__)
 
 struct os_cpu_usage_info {
@@ -96,20 +167,20 @@ struct os_cpu_usage_info {
 os_cpu_usage_info_t *os_cpu_usage_info_start(void)
 {
 	struct os_cpu_usage_info *info = bmalloc(sizeof(*info));
-	struct tms               time_sample;
+	struct tms time_sample;
 
-	info->last_cpu_time  = times(&time_sample);
-	info->last_sys_time  = time_sample.tms_stime;
+	info->last_cpu_time = times(&time_sample);
+	info->last_sys_time = time_sample.tms_stime;
 	info->last_user_time = time_sample.tms_utime;
-	info->core_count     = sysconf(_SC_NPROCESSORS_ONLN);
+	info->core_count = sysconf(_SC_NPROCESSORS_ONLN);
 	return info;
 }
 
 double os_cpu_usage_info_query(os_cpu_usage_info_t *info)
 {
 	struct tms time_sample;
-	clock_t    cur_cpu_time;
-	double     percent;
+	clock_t cur_cpu_time;
+	double percent;
 
 	if (!info)
 		return 0.0;
@@ -121,12 +192,12 @@ double os_cpu_usage_info_query(os_cpu_usage_info_t *info)
 		return 0.0;
 
 	percent = (double)(time_sample.tms_stime - info->last_sys_time +
-			(time_sample.tms_utime - info->last_user_time));
+			   (time_sample.tms_utime - info->last_user_time));
 	percent /= (double)(cur_cpu_time - info->last_cpu_time);
 	percent /= (double)info->core_count;
 
-	info->last_cpu_time  = cur_cpu_time;
-	info->last_sys_time  = time_sample.tms_stime;
+	info->last_cpu_time = cur_cpu_time;
+	info->last_sys_time = time_sample.tms_stime;
 	info->last_user_time = time_sample.tms_utime;
 
 	return percent * 100.0;
@@ -151,8 +222,8 @@ bool os_sleepto_ns(uint64_t time_target)
 	struct timespec req, remain;
 	memset(&req, 0, sizeof(req));
 	memset(&remain, 0, sizeof(remain));
-	req.tv_sec = time_target/1000000000;
-	req.tv_nsec = time_target%1000000000;
+	req.tv_sec = time_target / 1000000000;
+	req.tv_nsec = time_target % 1000000000;
 
 	while (nanosleep(&req, &remain)) {
 		req = remain;
@@ -162,9 +233,26 @@ bool os_sleepto_ns(uint64_t time_target)
 	return true;
 }
 
+bool os_sleepto_ns_fast(uint64_t time_target)
+{
+	uint64_t current = os_gettime_ns();
+	if (time_target < current)
+		return false;
+
+	do {
+		uint64_t remain_us = (time_target - current + 999) / 1000;
+		useconds_t us = remain_us >= 1000000 ? 999999 : remain_us;
+		usleep(us);
+
+		current = os_gettime_ns();
+	} while (time_target > current);
+
+	return true;
+}
+
 void os_sleep_ms(uint32_t duration)
 {
-	usleep(duration*1000);
+	usleep(duration * 1000);
 }
 
 #if !defined(__APPLE__)
@@ -173,7 +261,7 @@ uint64_t os_gettime_ns(void)
 {
 	struct timespec ts;
 	clock_gettime(CLOCK_MONOTONIC, &ts);
-	return ((uint64_t) ts.tv_sec * 1000000000ULL + (uint64_t) ts.tv_nsec);
+	return ((uint64_t)ts.tv_sec * 1000000000ULL + (uint64_t)ts.tv_nsec);
 }
 
 /* should return $HOME/.[name], or when using XDG,
@@ -258,13 +346,119 @@ int os_get_program_data_path(char *dst, size_t size, const char *name)
 
 char *os_get_program_data_path_ptr(const char *name)
 {
-	size_t len = snprintf(NULL, 0, "/usr/local/share/%s", !!name ? name : "");
+	size_t len =
+		snprintf(NULL, 0, "/usr/local/share/%s", !!name ? name : "");
 	char *str = bmalloc(len + 1);
 	snprintf(str, len + 1, "/usr/local/share/%s", !!name ? name : "");
 	str[len] = 0;
 	return str;
 }
 
+#if defined(__OpenBSD__)
+// a bit modified version of https://stackoverflow.com/a/31495527
+ssize_t os_openbsd_get_executable_path(char *epath)
+{
+	int mib[4];
+	char **argv;
+	size_t len;
+	const char *comm;
+	int ok = 0;
+
+	mib[0] = CTL_KERN;
+	mib[1] = KERN_PROC_ARGS;
+	mib[2] = getpid();
+	mib[3] = KERN_PROC_ARGV;
+
+	if (sysctl(mib, 4, NULL, &len, NULL, 0) < 0)
+		abort();
+
+	if (!(argv = malloc(len)))
+		abort();
+
+	if (sysctl(mib, 4, argv, &len, NULL, 0) < 0)
+		abort();
+
+	comm = argv[0];
+
+	if (*comm == '/' || *comm == '.') {
+		if (realpath(comm, epath))
+			ok = 1;
+	} else {
+		char *sp;
+		char *xpath = strdup(getenv("PATH"));
+		char *path = strtok_r(xpath, ":", &sp);
+		struct stat st;
+
+		if (!xpath)
+			abort();
+
+		while (path) {
+			snprintf(epath, PATH_MAX, "%s/%s", path, comm);
+
+			if (!stat(epath, &st) && (st.st_mode & S_IXUSR)) {
+				ok = 1;
+				break;
+			}
+			path = strtok_r(NULL, ":", &sp);
+		}
+
+		free(xpath);
+	}
+
+	free(argv);
+	return ok ? (ssize_t)strlen(epath) : -1;
+}
+#endif
+
+char *os_get_executable_path_ptr(const char *name)
+{
+	char exe[PATH_MAX];
+#if defined(__FreeBSD__) || defined(__DragonFly__)
+	int sysctlname[4] = {CTL_KERN, KERN_PROC, KERN_PROC_PATHNAME, -1};
+	size_t pathlen = PATH_MAX;
+	ssize_t count;
+	if (sysctl(sysctlname, nitems(sysctlname), exe, &pathlen, NULL, 0) ==
+	    -1) {
+		blog(LOG_ERROR, "sysctl(KERN_PROC_PATHNAME) failed, errno %d",
+		     errno);
+		return NULL;
+	}
+	count = pathlen;
+#elif defined(__OpenBSD__)
+	ssize_t count = os_openbsd_get_executable_path(exe);
+#else
+	ssize_t count = readlink("/proc/self/exe", exe, PATH_MAX - 1);
+	if (count >= 0) {
+		exe[count] = '\0';
+	}
+#endif
+	const char *path_out = NULL;
+	struct dstr path;
+
+	if (count == -1) {
+		return NULL;
+	}
+
+	path_out = dirname(exe);
+	if (!path_out) {
+		return NULL;
+	}
+
+	dstr_init_copy(&path, path_out);
+	dstr_cat(&path, "/");
+
+	if (name && *name) {
+		dstr_cat(&path, name);
+	}
+
+	return path.array;
+}
+
+bool os_get_emulation_status(void)
+{
+	return false;
+}
+
 #endif
 
 bool os_file_exists(const char *path)
@@ -301,23 +495,23 @@ char *os_get_abs_path_ptr(const char *path)
 }
 
 struct os_dir {
-	const char       *path;
-	DIR              *dir;
-	struct dirent    *cur_dirent;
+	const char *path;
+	DIR *dir;
+	struct dirent *cur_dirent;
 	struct os_dirent out;
 };
 
 os_dir_t *os_opendir(const char *path)
 {
 	struct os_dir *dir;
-	DIR           *dir_val;
+	DIR *dir_val;
 
 	dir_val = opendir(path);
 	if (!dir_val)
 		return NULL;
 
 	dir = bzalloc(sizeof(struct os_dir));
-	dir->dir  = dir_val;
+	dir->dir = dir_val;
 	dir->path = path;
 	return dir;
 }
@@ -336,13 +530,17 @@ struct os_dirent *os_readdir(os_dir_t *dir)
 {
 	struct dstr file_path = {0};
 
-	if (!dir) return NULL;
+	if (!dir)
+		return NULL;
 
 	dir->cur_dirent = readdir(dir->dir);
 	if (!dir->cur_dirent)
 		return NULL;
 
-	strncpy(dir->out.d_name, dir->cur_dirent->d_name, 255);
+	const size_t length = strlen(dir->cur_dirent->d_name);
+	if (sizeof(dir->out.d_name) <= length)
+		return NULL;
+	memcpy(dir->out.d_name, dir->cur_dirent->d_name, length + 1);
 
 	dstr_copy(&file_path, dir->path);
 	dstr_cat(&file_path, "/");
@@ -411,7 +609,7 @@ int os_glob(const char *pattern, int flags, os_glob_t **pglob)
 void os_globfree(os_glob_t *pglob)
 {
 	if (pglob) {
-		struct posix_glob_info *pgi = (struct posix_glob_info*)pglob;
+		struct posix_glob_info *pgi = (struct posix_glob_info *)pglob;
 		globfree(&pgi->gl);
 
 		bfree(pgi->base.gl_pathv);
@@ -508,18 +706,25 @@ int os_chdir(const char *path)
 
 #if !defined(__APPLE__)
 
-#if HAVE_DBUS
+#if defined(GIO_FOUND)
 struct dbus_sleep_info;
+struct portal_inhibit_info;
 
 extern struct dbus_sleep_info *dbus_sleep_info_create(void);
 extern void dbus_inhibit_sleep(struct dbus_sleep_info *dbus, const char *sleep,
-		bool active);
+			       bool active);
 extern void dbus_sleep_info_destroy(struct dbus_sleep_info *dbus);
+
+extern struct portal_inhibit_info *portal_inhibit_info_create(void);
+extern void portal_inhibit(struct portal_inhibit_info *portal,
+			   const char *reason, bool active);
+extern void portal_inhibit_info_destroy(struct portal_inhibit_info *portal);
 #endif
 
 struct os_inhibit_info {
-#if HAVE_DBUS
+#if defined(GIO_FOUND)
 	struct dbus_sleep_info *dbus;
+	struct portal_inhibit_info *portal;
 #endif
 	pthread_t screensaver_thread;
 	os_event_t *stop_event;
@@ -533,8 +738,10 @@ os_inhibit_t *os_inhibit_sleep_create(const char *reason)
 	struct os_inhibit_info *info = bzalloc(sizeof(*info));
 	sigset_t set;
 
-#if HAVE_DBUS
-	info->dbus = dbus_sleep_info_create();
+#if defined(GIO_FOUND)
+	info->portal = portal_inhibit_info_create();
+	if (!info->portal)
+		info->dbus = dbus_sleep_info_create();
 #endif
 
 	os_event_init(&info->stop_event, OS_EVENT_TYPE_AUTO);
@@ -544,8 +751,8 @@ os_inhibit_t *os_inhibit_sleep_create(const char *reason)
 	posix_spawnattr_setsigmask(&info->attr, &set);
 	sigaddset(&set, SIGPIPE);
 	posix_spawnattr_setsigdefault(&info->attr, &set);
-	posix_spawnattr_setflags(&info->attr,
-			POSIX_SPAWN_SETSIGDEF | POSIX_SPAWN_SETSIGMASK);
+	posix_spawnattr_setflags(&info->attr, POSIX_SPAWN_SETSIGDEF |
+						      POSIX_SPAWN_SETSIGMASK);
 
 	info->reason = bstrdup(reason);
 	return info;
@@ -555,14 +762,15 @@ extern char **environ;
 
 static void reset_screensaver(os_inhibit_t *info)
 {
-	char *argv[3] = {(char*)"xdg-screensaver", (char*)"reset", NULL};
+	char *argv[3] = {(char *)"xdg-screensaver", (char *)"reset", NULL};
 	pid_t pid;
 
-	int err = posix_spawnp(&pid, "xdg-screensaver", NULL, &info->attr,
-			argv, environ);
+	int err = posix_spawnp(&pid, "xdg-screensaver", NULL, &info->attr, argv,
+			       environ);
 	if (err == 0) {
 		int status;
-		while (waitpid(pid, &status, 0) == -1);
+		while (waitpid(pid, &status, 0) == -1)
+			;
 	} else {
 		blog(LOG_WARNING, "Failed to create xdg-screensaver: %d", err);
 	}
@@ -587,7 +795,9 @@ bool os_inhibit_sleep_set_active(os_inhibit_t *info, bool active)
 	if (info->active == active)
 		return false;
 
-#if HAVE_DBUS
+#if defined(GIO_FOUND)
+	if (info->portal)
+		portal_inhibit(info->portal, info->reason, active);
 	if (info->dbus)
 		dbus_inhibit_sleep(info->dbus, info->reason, active);
 #endif
@@ -597,10 +807,10 @@ bool os_inhibit_sleep_set_active(os_inhibit_t *info, bool active)
 
 	if (active) {
 		ret = pthread_create(&info->screensaver_thread, NULL,
-				&screensaver_thread, info);
+				     &screensaver_thread, info);
 		if (ret < 0) {
 			blog(LOG_ERROR, "Failed to create screensaver "
-			                "inhibitor thread");
+					"inhibitor thread");
 			return false;
 		}
 	} else {
@@ -616,7 +826,8 @@ void os_inhibit_sleep_destroy(os_inhibit_t *info)
 {
 	if (info) {
 		os_inhibit_sleep_set_active(info, false);
-#if HAVE_DBUS
+#if defined(GIO_FOUND)
+		portal_inhibit_info_destroy(info->portal);
 		dbus_sleep_info_destroy(info->dbus);
 #endif
 		os_event_destroy(info->stop_event);
@@ -683,8 +894,8 @@ static void os_get_cores_internal(void)
 				continue;
 
 			if (dstr_is_empty(&proc_phys_ids) ||
-				(!dstr_is_empty(&proc_phys_ids) &&
-				!dstr_find(&proc_phys_ids, proc_phys_id.array))) {
+			    (!dstr_is_empty(&proc_phys_ids) &&
+			     !dstr_find(&proc_phys_ids, proc_phys_id.array))) {
 				dstr_cat_dstr(&proc_phys_ids, &proc_phys_id);
 				dstr_cat(&proc_phys_ids, " ");
 				core_count += atoi(start);
@@ -743,7 +954,7 @@ static void os_get_cores_internal(void)
 	len = strcspn(core_count, " ");
 	dstr_ncopy(&proc_cores, core_count, len);
 
-	FreeBSD_cores_cleanup:
+FreeBSD_cores_cleanup:
 	if (!dstr_is_empty(&proc_packages))
 		packages = atoi(proc_packages.array);
 	if (!dstr_is_empty(&proc_cores))
@@ -783,8 +994,8 @@ uint64_t os_get_sys_free_size(void)
 {
 	uint64_t mem_free = 0;
 	size_t length = sizeof(mem_free);
-	if (sysctlbyname("vm.stats.vm.v_free_count", &mem_free, &length,
-				NULL, 0) < 0)
+	if (sysctlbyname("vm.stats.vm.v_free_count", &mem_free, &length, NULL,
+			 0) < 0)
 		return 0;
 	return mem_free;
 }
@@ -793,8 +1004,8 @@ static inline bool os_get_proc_memory_usage_internal(struct kinfo_proc *kinfo)
 {
 	int mib[] = {CTL_KERN, KERN_PROC, KERN_PROC_PID, getpid()};
 	size_t length = sizeof(*kinfo);
-	if (sysctl(mib, sizeof(mib) / sizeof(mib[0]), kinfo, &length,
-				NULL, 0) < 0)
+	if (sysctl(mib, sizeof(mib) / sizeof(mib[0]), kinfo, &length, NULL, 0) <
+	    0)
 		return false;
 	return true;
 }
@@ -806,8 +1017,8 @@ bool os_get_proc_memory_usage(os_proc_memory_usage_t *usage)
 		return false;
 
 	usage->resident_size =
-			(uint64_t)kinfo.ki_rssize * sysconf(_SC_PAGESIZE);
-	usage->virtual_size  = (uint64_t)kinfo.ki_size;
+		(uint64_t)kinfo.ki_rssize * sysconf(_SC_PAGESIZE);
+	usage->virtual_size = (uint64_t)kinfo.ki_size;
 	return true;
 }
 
@@ -827,10 +1038,12 @@ uint64_t os_get_proc_virtual_size(void)
 	return (uint64_t)kinfo.ki_size;
 }
 #else
-uint64_t os_get_sys_free_size(void) {return 0;}
-
-typedef struct
+uint64_t os_get_sys_free_size(void)
 {
+	return 0;
+}
+
+typedef struct {
 	unsigned long virtual_size;
 	unsigned long resident_size;
 	unsigned long share_pages;
@@ -848,14 +1061,9 @@ static inline bool os_get_proc_memory_usage_internal(statm_t *statm)
 	if (!f)
 		return false;
 
-	if (fscanf(f, "%lu %lu %lu %lu %lu %lu %lu",
-	    &statm->virtual_size,
-	    &statm->resident_size,
-	    &statm->share_pages,
-	    &statm->text,
-	    &statm->library,
-	    &statm->data,
-	    &statm->dirty_pages) != 7) {
+	if (fscanf(f, "%lu %lu %lu %lu %lu %lu %lu", &statm->virtual_size,
+		   &statm->resident_size, &statm->share_pages, &statm->text,
+		   &statm->library, &statm->data, &statm->dirty_pages) != 7) {
 		fclose(f);
 		return false;
 	}
@@ -870,8 +1078,9 @@ bool os_get_proc_memory_usage(os_proc_memory_usage_t *usage)
 	if (!os_get_proc_memory_usage_internal(&statm))
 		return false;
 
-	usage->resident_size = statm.resident_size;
-	usage->virtual_size  = statm.virtual_size;
+	usage->resident_size =
+		(uint64_t)statm.resident_size * sysconf(_SC_PAGESIZE);
+	usage->virtual_size = statm.virtual_size;
 	return true;
 }
 
@@ -880,7 +1089,7 @@ uint64_t os_get_proc_resident_size(void)
 	statm_t statm = {};
 	if (!os_get_proc_memory_usage_internal(&statm))
 		return 0;
-	return (uint64_t)statm.resident_size;
+	return (uint64_t)statm.resident_size * sysconf(_SC_PAGESIZE);
 }
 
 uint64_t os_get_proc_virtual_size(void)
