@@ -212,7 +212,8 @@ bool QSV_Encoder_Internal::InitParams(qsv_param_t *pParams)
 	m_mfxEncParams.mfx.GopRefDist = pParams->nbFrames + 1;
 
 	enum qsv_cpu_platform qsv_platform = qsv_get_cpu_platform();
-	if ((qsv_platform >= QSV_CPU_PLATFORM_ICL) &&
+	if ((qsv_platform >= QSV_CPU_PLATFORM_ICL ||
+	     qsv_platform == QSV_CPU_PLATFORM_UNKNOWN) &&
 	    (pParams->nbFrames == 0) &&
 	    (m_ver.Major == 1 && m_ver.Minor >= 31)) {
 		m_mfxEncParams.mfx.LowPower = MFX_CODINGOPTION_ON;
