@@ -74,6 +74,7 @@ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 	blog(level, "[qsv encoder: '%s'] " format, \
 	     obs_encoder_get_name(obsqsv->encoder), ##__VA_ARGS__)
 
+#define error(format, ...) do_log(LOG_ERROR, format, ##__VA_ARGS__)
 #define warn(format, ...) do_log(LOG_WARNING, format, ##__VA_ARGS__)
 #define info(format, ...) do_log(LOG_INFO, format, ##__VA_ARGS__)
 #define debug(format, ...) do_log(LOG_DEBUG, format, ##__VA_ARGS__)
@@ -99,7 +100,7 @@ struct obs_qsv {
 
 /* ------------------------------------------------------------------------- */
 
-static CRITICAL_SECTION g_QsvCs;
+static SRWLOCK g_QsvLock = SRWLOCK_INIT;
 static unsigned short g_verMajor;
 static unsigned short g_verMinor;
 static int64_t g_pts2dtsShift;
@@ -117,10 +118,10 @@ static void obs_qsv_stop(void *data);
 static void clear_data(struct obs_qsv *obsqsv)
 {
 	if (obsqsv->context) {
-		EnterCriticalSection(&g_QsvCs);
+		AcquireSRWLockExclusive(&g_QsvLock);
 		qsv_encoder_close(obsqsv->context);
 		obsqsv->context = NULL;
-		LeaveCriticalSection(&g_QsvCs);
+		ReleaseSRWLockExclusive(&g_QsvLock);
 
 		// bfree(obsqsv->sei);
 		bfree(obsqsv->extra_data);
@@ -188,7 +189,8 @@ static inline void add_strings(obs_property_t *list, const char *const *strings)
 static inline bool is_skl_or_greater_platform()
 {
 	enum qsv_cpu_platform plat = qsv_get_cpu_platform();
-	return (plat >= QSV_CPU_PLATFORM_SKL);
+	return (plat >= QSV_CPU_PLATFORM_SKL ||
+		plat == QSV_CPU_PLATFORM_UNKNOWN);
 }
 
 static bool update_latency(obs_data_t *settings)
@@ -313,7 +315,8 @@ static bool profile_modified(obs_properties_t *ppts, obs_property_t *p,
 	const char *profile = obs_data_get_string(settings, "profile");
 	enum qsv_cpu_platform plat = qsv_get_cpu_platform();
 	bool bVisible = ((astrcmpi(profile, "high") == 0) &&
-			 (plat >= QSV_CPU_PLATFORM_ICL));
+			 (plat >= QSV_CPU_PLATFORM_ICL ||
+			  plat == QSV_CPU_PLATFORM_UNKNOWN));
 	p = obs_properties_get(ppts, "CQM");
 	obs_property_set_visible(p, bVisible);
 	return true;
@@ -324,7 +327,9 @@ static inline void add_rate_controls(obs_property_t *list,
 {
 	enum qsv_cpu_platform plat = qsv_get_cpu_platform();
 	while (rc->name) {
-		if (!rc->haswell_or_greater || plat >= QSV_CPU_PLATFORM_HSW)
+		if (!rc->haswell_or_greater ||
+		    (plat >= QSV_CPU_PLATFORM_HSW ||
+		     plat == QSV_CPU_PLATFORM_UNKNOWN))
 			obs_property_list_add_string(list, rc->name, rc->name);
 		rc++;
 	}
@@ -335,36 +340,37 @@ static obs_properties_t *obs_qsv_props(void *unused)
 	UNUSED_PARAMETER(unused);
 
 	obs_properties_t *props = obs_properties_create();
-	obs_property_t *list;
+	obs_property_t *prop;
 
-	list = obs_properties_add_list(props, "target_usage", TEXT_SPEED,
+	prop = obs_properties_add_list(props, "target_usage", TEXT_SPEED,
 				       OBS_COMBO_TYPE_LIST,
 				       OBS_COMBO_FORMAT_STRING);
-	add_strings(list, qsv_usage_names);
+	add_strings(prop, qsv_usage_names);
 
-	list = obs_properties_add_list(props, "profile", TEXT_PROFILE,
+	prop = obs_properties_add_list(props, "profile", TEXT_PROFILE,
 				       OBS_COMBO_TYPE_LIST,
 				       OBS_COMBO_FORMAT_STRING);
-	add_strings(list, qsv_profile_names);
+	add_strings(prop, qsv_profile_names);
 
-	obs_property_set_modified_callback(list, profile_modified);
+	obs_property_set_modified_callback(prop, profile_modified);
 
-	obs_properties_add_int(props, "keyint_sec", TEXT_KEYINT_SEC, 1, 20, 1);
+	prop = obs_properties_add_int(props, "keyint_sec", TEXT_KEYINT_SEC, 1,
+				      20, 1);
+	obs_property_int_set_suffix(prop, " s");
 
-	list = obs_properties_add_list(props, "rate_control", TEXT_RATE_CONTROL,
+	prop = obs_properties_add_list(props, "rate_control", TEXT_RATE_CONTROL,
 				       OBS_COMBO_TYPE_LIST,
 				       OBS_COMBO_FORMAT_STRING);
-	add_rate_controls(list, qsv_ratecontrols);
-	obs_property_set_modified_callback(list, rate_control_modified);
+	add_rate_controls(prop, qsv_ratecontrols);
+	obs_property_set_modified_callback(prop, rate_control_modified);
 
-	obs_property_t *p;
-	p = obs_properties_add_int(props, "bitrate", TEXT_TARGET_BITRATE, 50,
-				   10000000, 50);
-	obs_property_int_set_suffix(p, " Kbps");
+	prop = obs_properties_add_int(props, "bitrate", TEXT_TARGET_BITRATE, 50,
+				      10000000, 50);
+	obs_property_int_set_suffix(prop, " Kbps");
 
-	p = obs_properties_add_int(props, "max_bitrate", TEXT_MAX_BITRATE, 50,
-				   10000000, 50);
-	obs_property_int_set_suffix(p, " Kbps");
+	prop = obs_properties_add_int(props, "max_bitrate", TEXT_MAX_BITRATE,
+				      50, 10000000, 50);
+	obs_property_int_set_suffix(prop, " Kbps");
 
 	obs_properties_add_int(props, "accuracy", TEXT_ACCURACY, 0, 10000, 1);
 	obs_properties_add_int(props, "convergence", TEXT_CONVERGENCE, 0, 10,
@@ -374,11 +380,11 @@ static obs_properties_t *obs_qsv_props(void *unused)
 	obs_properties_add_int(props, "qpb", "QPB", 1, 51, 1);
 	obs_properties_add_int(props, "icq_quality", TEXT_ICQ_QUALITY, 1, 51,
 			       1);
-	list = obs_properties_add_list(props, "latency", TEXT_LATENCY,
+	prop = obs_properties_add_list(props, "latency", TEXT_LATENCY,
 				       OBS_COMBO_TYPE_LIST,
 				       OBS_COMBO_FORMAT_STRING);
-	add_strings(list, qsv_latency_names);
-	obs_property_set_long_description(list,
+	add_strings(prop, qsv_latency_names);
+	obs_property_set_long_description(prop,
 					  obs_module_text("Latency.ToolTip"));
 
 	obs_properties_add_int(props, "bframes", TEXT_BFRAMES, 0, 3, 1);
@@ -598,13 +604,13 @@ static bool obs_qsv_update(void *data, obs_data_t *settings)
 	int ret;
 
 	if (success) {
-		EnterCriticalSection(&g_QsvCs);
+		AcquireSRWLockExclusive(&g_QsvLock);
 
 		ret = qsv_encoder_reconfig(obsqsv->context, &obsqsv->params);
 		if (ret != 0)
 			warn("Failed to reconfigure: %d", ret);
 
-		LeaveCriticalSection(&g_QsvCs);
+		ReleaseSRWLockExclusive(&g_QsvLock);
 
 		return ret == 0;
 	}
@@ -614,15 +620,36 @@ static bool obs_qsv_update(void *data, obs_data_t *settings)
 
 static void *obs_qsv_create(obs_data_t *settings, obs_encoder_t *encoder)
 {
-	InitializeCriticalSection(&g_QsvCs);
-
 	struct obs_qsv *obsqsv = bzalloc(sizeof(struct obs_qsv));
 	obsqsv->encoder = encoder;
 
+	video_t *video = obs_encoder_video(encoder);
+	const struct video_output_info *voi = video_output_get_info(video);
+	switch (voi->format) {
+	case VIDEO_FORMAT_I010:
+	case VIDEO_FORMAT_P010:
+		const char *const text = obs_module_text("10bitUnsupportedAvc");
+		obs_encoder_set_last_error(encoder, text);
+		error("%s", text);
+		bfree(obsqsv);
+		return NULL;
+	default:
+		switch (voi->colorspace) {
+		case VIDEO_CS_2100_PQ:
+		case VIDEO_CS_2100_HLG:
+			const char *const text =
+				obs_module_text("8bitUnsupportedHdr");
+			obs_encoder_set_last_error(encoder, text);
+			error("%s", text);
+			bfree(obsqsv);
+			return NULL;
+		}
+	}
+
 	if (update_settings(obsqsv, settings)) {
-		EnterCriticalSection(&g_QsvCs);
+		AcquireSRWLockExclusive(&g_QsvLock);
 		obsqsv->context = qsv_encoder_open(&obsqsv->params);
-		LeaveCriticalSection(&g_QsvCs);
+		ReleaseSRWLockExclusive(&g_QsvLock);
 
 		if (obsqsv->context == NULL)
 			warn("qsv failed to load");
@@ -775,10 +802,6 @@ static bool obs_qsv_sei(void *data, uint8_t **sei, size_t *size)
 	if (!obsqsv->context)
 		return false;
 
-	/* (Jim) Unused */
-	UNUSED_PARAMETER(sei);
-	UNUSED_PARAMETER(size);
-
 	*sei = obsqsv->sei;
 	*size = obsqsv->sei_size;
 	return true;
@@ -799,7 +822,8 @@ static inline void cap_resolution(obs_encoder_t *encoder,
 	info->height = height;
 	info->width = width;
 
-	if (qsv_platform <= QSV_CPU_PLATFORM_IVB) {
+	if (qsv_platform <= QSV_CPU_PLATFORM_IVB &&
+	    qsv_platform != QSV_CPU_PLATFORM_UNKNOWN) {
 		if (width > 1920) {
 			info->width = 1920;
 		}
@@ -826,8 +850,23 @@ static void obs_qsv_video_info(void *data, struct video_scale_info *info)
 	cap_resolution(obsqsv->encoder, info);
 }
 
+static mfxU64 ts_obs_to_mfx(int64_t ts, const struct video_output_info *voi)
+{
+	return ts * 90000 / voi->fps_num;
+}
+
+static int64_t ts_mfx_to_obs(mfxI64 ts, const struct video_output_info *voi)
+{
+	int64_t div = 90000 * (int64_t)voi->fps_den;
+	/* Round to the nearest integer multiple of `voi->fps_den`. */
+	if (ts < 0)
+		return (ts * voi->fps_num - div / 2) / div * voi->fps_den;
+	else
+		return (ts * voi->fps_num + div / 2) / div * voi->fps_den;
+}
+
 static void parse_packet(struct obs_qsv *obsqsv, struct encoder_packet *packet,
-			 mfxBitstream *pBS, uint32_t fps_num,
+			 mfxBitstream *pBS, const struct video_output_info *voi,
 			 bool *received_packet)
 {
 	uint8_t *start, *end;
@@ -845,7 +884,7 @@ static void parse_packet(struct obs_qsv *obsqsv, struct encoder_packet *packet,
 	packet->data = obsqsv->packet_data.array;
 	packet->size = obsqsv->packet_data.num;
 	packet->type = OBS_ENCODER_VIDEO;
-	packet->pts = pBS->TimeStamp * fps_num / 90000;
+	packet->pts = ts_mfx_to_obs((mfxI64)pBS->TimeStamp, voi);
 	packet->keyframe = (pBS->FrameType & MFX_FRAMETYPE_IDR);
 
 	uint16_t frameType = pBS->FrameType;
@@ -906,7 +945,7 @@ static void parse_packet(struct obs_qsv *obsqsv, struct encoder_packet *packet,
 			g_prevDts = packet->dts;
 		}
 	} else {
-		packet->dts = pBS->DecodeTimeStamp * fps_num / 90000;
+		packet->dts = ts_mfx_to_obs(pBS->DecodeTimeStamp, voi);
 	}
 
 #if 0
@@ -931,7 +970,7 @@ static bool obs_qsv_encode(void *data, struct encoder_frame *frame,
 	if (!frame || !packet || !received_packet)
 		return false;
 
-	EnterCriticalSection(&g_QsvCs);
+	AcquireSRWLockExclusive(&g_QsvLock);
 
 	video_t *video = obs_encoder_video(obsqsv->encoder);
 	const struct video_output_info *voi = video_output_get_info(video);
@@ -940,7 +979,7 @@ static bool obs_qsv_encode(void *data, struct encoder_frame *frame,
 
 	int ret;
 
-	mfxU64 qsvPTS = frame->pts * 90000 / voi->fps_num;
+	mfxU64 qsvPTS = ts_obs_to_mfx(frame->pts, voi);
 
 	// FIXME: remove null check from the top of this function
 	// if we actually do expect null frames to complete output.
@@ -955,13 +994,13 @@ static bool obs_qsv_encode(void *data, struct encoder_frame *frame,
 
 	if (ret < 0) {
 		warn("encode failed");
-		LeaveCriticalSection(&g_QsvCs);
+		ReleaseSRWLockExclusive(&g_QsvLock);
 		return false;
 	}
 
-	parse_packet(obsqsv, packet, pBS, voi->fps_num, received_packet);
+	parse_packet(obsqsv, packet, pBS, voi, received_packet);
 
-	LeaveCriticalSection(&g_QsvCs);
+	ReleaseSRWLockExclusive(&g_QsvLock);
 
 	return true;
 }
@@ -982,7 +1021,7 @@ static bool obs_qsv_encode_tex(void *data, uint32_t handle, int64_t pts,
 	if (!packet || !received_packet)
 		return false;
 
-	EnterCriticalSection(&g_QsvCs);
+	AcquireSRWLockExclusive(&g_QsvLock);
 
 	video_t *video = obs_encoder_video(obsqsv->encoder);
 	const struct video_output_info *voi = video_output_get_info(video);
@@ -991,20 +1030,20 @@ static bool obs_qsv_encode_tex(void *data, uint32_t handle, int64_t pts,
 
 	int ret;
 
-	mfxU64 qsvPTS = pts * 90000 / voi->fps_num;
+	mfxU64 qsvPTS = ts_obs_to_mfx(pts, voi);
 
 	ret = qsv_encoder_encode_tex(obsqsv->context, qsvPTS, handle, lock_key,
 				     next_key, &pBS);
 
 	if (ret < 0) {
 		warn("encode failed");
-		LeaveCriticalSection(&g_QsvCs);
+		ReleaseSRWLockExclusive(&g_QsvLock);
 		return false;
 	}
 
-	parse_packet(obsqsv, packet, pBS, voi->fps_num, received_packet);
+	parse_packet(obsqsv, packet, pBS, voi, received_packet);
 
-	LeaveCriticalSection(&g_QsvCs);
+	ReleaseSRWLockExclusive(&g_QsvLock);
 
 	return true;
 }
