@@ -3,7 +3,7 @@
 #include <inttypes.h>
 #include "graphics-hook.h"
 #include "../graphics-hook-ver.h"
-#include "../obfuscate.h"
+#include "../../libobs/util/windows/obfuscate.h"
 
 #define DEBUG_OUTPUT
 
@@ -319,7 +319,7 @@ static inline bool attempt_hook(void)
 	static bool d3d12_hooked = false;
 	static bool dxgi_hooked = false;
 	static bool gl_hooked = false;
-#if COMPILE_VULKAN_HOOK
+#ifdef COMPILE_VULKAN_HOOK
 	static bool vulkan_hooked = false;
 	if (!vulkan_hooked) {
 		vulkan_hooked = hook_vulkan();
@@ -329,7 +329,7 @@ static inline bool attempt_hook(void)
 	}
 #endif //COMPILE_VULKAN_HOOK
 
-#if COMPILE_D3D12_HOOK
+#ifdef COMPILE_D3D12_HOOK
 	if (!d3d12_hooked) {
 		d3d12_hooked = hook_d3d12();
 	}
@@ -931,7 +931,7 @@ __declspec(dllexport) LRESULT CALLBACK
 		HMODULE user32 = GetModuleHandleW(L"USER32");
 		BOOL(WINAPI * unhook_windows_hook_ex)(HHOOK) = NULL;
 
-		unhook_windows_hook_ex = get_obfuscated_func(
+		unhook_windows_hook_ex = ms_get_obfuscated_func(
 			user32, "VojeleY`bdgxvM`hhDz", 0x7F55F80C9EE3A213ULL);
 
 		if (unhook_windows_hook_ex)
