@@ -13,6 +13,8 @@
 #include <dxgi.h>
 #include <d3d11.h>
 
+#include "dxgi-helpers.hpp"
+
 #include "vulkan-capture.h"
 
 /* ======================================================================== */
@@ -113,6 +115,8 @@ struct vk_data {
 	ID3D11DeviceContext *d3d11_context;
 };
 
+__declspec(thread) int vk_presenting = 0;
+
 /* ------------------------------------------------------------------------- */
 
 static void *vk_alloc(const VkAllocationCallbacks *ac, size_t size,
@@ -631,12 +635,13 @@ static inline bool vk_shtex_init_d3d11_tex(struct vk_data *data,
 	desc.Height = height;
 	desc.MipLevels = 1;
 	desc.ArraySize = 1;
-	desc.Format = format;
+	desc.Format = apply_dxgi_format_typeless(
+		format, global_hook_info->allow_srgb_alias);
 	desc.SampleDesc.Count = 1;
 	desc.SampleDesc.Quality = 0;
 	desc.Usage = D3D11_USAGE_DEFAULT;
 	desc.MiscFlags = D3D11_RESOURCE_MISC_SHARED;
-	desc.BindFlags = D3D11_BIND_RENDER_TARGET | D3D11_BIND_SHADER_RESOURCE;
+	desc.BindFlags = D3D11_BIND_SHADER_RESOURCE;
 
 	hr = ID3D11Device_CreateTexture2D(data->d3d11_device, &desc, NULL,
 					  &swap->d3d11_tex);
@@ -748,24 +753,6 @@ static inline bool vk_shtex_init_vulkan_tex(struct vk_data *data,
 	VkPhysicalDeviceMemoryProperties pdmp;
 	ifuncs->GetPhysicalDeviceMemoryProperties(data->phy_device, &pdmp);
 
-	uint32_t mem_type_idx = 0;
-
-	for (; mem_type_idx < pdmp.memoryTypeCount; mem_type_idx++) {
-		if ((mr.memoryTypeBits & (1 << mem_type_idx)) &&
-		    (pdmp.memoryTypes[mem_type_idx].propertyFlags &
-		     VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT) ==
-			    VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT) {
-			break;
-		}
-	}
-
-	if (mem_type_idx == pdmp.memoryTypeCount) {
-		flog("failed to get memory type index");
-		funcs->DestroyImage(device, swap->export_image, data->ac);
-		swap->export_image = VK_NULL_HANDLE;
-		return false;
-	}
-
 	/* -------------------------------------------------------- */
 	/* allocate memory                                          */
 
@@ -781,7 +768,6 @@ static inline bool vk_shtex_init_vulkan_tex(struct vk_data *data,
 	mai.sType = VK_STRUCTURE_TYPE_MEMORY_ALLOCATE_INFO;
 	mai.pNext = &imw32hi;
 	mai.allocationSize = mr.size;
-	mai.memoryTypeIndex = mem_type_idx;
 
 	VkMemoryDedicatedAllocateInfo mdai;
 	mdai.sType = VK_STRUCTURE_TYPE_MEMORY_DEDICATED_ALLOCATE_INFO;
@@ -794,9 +780,46 @@ static inline bool vk_shtex_init_vulkan_tex(struct vk_data *data,
 		imw32hi.pNext = &mdai;
 	}
 
-	res = funcs->AllocateMemory(device, &mai, NULL, &swap->export_mem);
-	if (VK_SUCCESS != res) {
-		flog("failed to AllocateMemory: %s", result_to_str(res));
+	bool allocated = false;
+	for (uint32_t i = 0; i < pdmp.memoryTypeCount; ++i) {
+		if ((mr.memoryTypeBits & (1 << i)) &&
+		    (pdmp.memoryTypes[i].propertyFlags &
+		     VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT) ==
+			    VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT) {
+			mai.memoryTypeIndex = i;
+			res = funcs->AllocateMemory(device, &mai, NULL,
+						    &swap->export_mem);
+			allocated = res == VK_SUCCESS;
+			if (allocated)
+				break;
+
+			flog("failed to AllocateMemory (DEVICE_LOCAL): %s (%d)",
+			     result_to_str(res), (int)res);
+		}
+	}
+
+	if (!allocated) {
+		/* Try again without DEVICE_LOCAL */
+		for (uint32_t i = 0; i < pdmp.memoryTypeCount; ++i) {
+			if ((mr.memoryTypeBits & (1 << i)) &&
+			    (pdmp.memoryTypes[i].propertyFlags &
+			     VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT) !=
+				    VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT) {
+				mai.memoryTypeIndex = i;
+				res = funcs->AllocateMemory(device, &mai, NULL,
+							    &swap->export_mem);
+				allocated = res == VK_SUCCESS;
+				if (allocated)
+					break;
+
+				flog("failed to AllocateMemory (not DEVICE_LOCAL): %s (%d)",
+				     result_to_str(res), (int)res);
+			}
+		}
+	}
+
+	if (!allocated) {
+		flog("failed to allocate memory of any type");
 		funcs->DestroyImage(device, swap->export_image, data->ac);
 		swap->export_image = VK_NULL_HANDLE;
 		return false;
@@ -1213,7 +1236,14 @@ static VkResult VKAPI_CALL OBS_QueuePresentKHR(VkQueue queue,
 		vk_capture(data, queue, info);
 	}
 
-	return funcs->QueuePresentKHR(queue, info);
+	if (vk_presenting != 0) {
+		flog("non-zero vk_presenting: %d", vk_presenting);
+	}
+
+	vk_presenting++;
+	VkResult res = funcs->QueuePresentKHR(queue, info);
+	vk_presenting--;
+	return res;
 }
 
 /* ======================================================================== */
