@@ -47,6 +47,7 @@ using namespace DShow;
 #define COLOR_RANGE       "color_range"
 #define DEACTIVATE_WNS    "deactivate_when_not_showing"
 #define AUTOROTATION      "autorotation"
+#define HW_DECODE         "hw_decode"
 
 #define TEXT_INPUT_NAME     obs_module_text("VideoCaptureDevice")
 #define TEXT_DEVICE         obs_module_text("Device")
@@ -66,6 +67,7 @@ using namespace DShow;
 #define TEXT_BUFFERING_OFF  obs_module_text("Buffering.Disable")
 #define TEXT_FLIP_IMAGE     obs_module_text("FlipVertically")
 #define TEXT_AUTOROTATION   obs_module_text("Autorotation")
+#define TEXT_HW_DECODE      obs_module_text("HardwareDecode")
 #define TEXT_AUDIO_MODE     obs_module_text("AudioOutputMode")
 #define TEXT_MODE_CAPTURE   obs_module_text("AudioOutputMode.Capture")
 #define TEXT_MODE_DSOUND    obs_module_text("AudioOutputMode.DirectSound")
@@ -76,6 +78,10 @@ using namespace DShow;
 #define TEXT_DEACTIVATE     obs_module_text("Deactivate")
 #define TEXT_COLOR_SPACE    obs_module_text("ColorSpace")
 #define TEXT_COLOR_DEFAULT  obs_module_text("ColorSpace.Default")
+#define TEXT_COLOR_709      obs_module_text("ColorSpace.709")
+#define TEXT_COLOR_601      obs_module_text("ColorSpace.601")
+#define TEXT_COLOR_2100PQ   obs_module_text("ColorSpace.2100PQ")
+#define TEXT_COLOR_2100HLG  obs_module_text("ColorSpace.2100HLG")
 #define TEXT_COLOR_RANGE    obs_module_text("ColorRange")
 #define TEXT_RANGE_DEFAULT  obs_module_text("ColorRange.Default")
 #define TEXT_RANGE_PARTIAL  obs_module_text("ColorRange.Partial")
@@ -184,6 +190,7 @@ struct DShowInput {
 	bool flip = false;
 	bool active = false;
 	bool autorotation = true;
+	bool hw_decode = false;
 
 	Decoder audio_decoder;
 	Decoder video_decoder;
@@ -191,7 +198,6 @@ struct DShowInput {
 	VideoConfig videoConfig;
 	AudioConfig audioConfig;
 
-	video_range_type range;
 	obs_source_frame2 frame;
 	obs_source_audio audio;
 	long lastRotation = 0;
@@ -272,6 +278,7 @@ struct DShowInput {
 	void OnEncodedAudioData(enum AVCodecID id, unsigned char *data,
 				size_t size, long long ts);
 
+	void OnReactivate();
 	void OnVideoData(const VideoConfig &config, unsigned char *data,
 			 size_t size, long long startTime, long long endTime,
 			 long rotation);
@@ -434,6 +441,8 @@ static inline video_format ConvertVideoFormat(VideoFormat format)
 		return VIDEO_FORMAT_UYVY;
 	case VideoFormat::HDYC:
 		return VIDEO_FORMAT_UYVY;
+	case VideoFormat::P010:
+		return VIDEO_FORMAT_P010;
 	default:
 		return VIDEO_FORMAT_NONE;
 	}
@@ -478,25 +487,18 @@ static inline enum speaker_layout convert_speaker_layout(uint8_t channels)
 //#define LOG_ENCODED_VIDEO_TS 1
 //#define LOG_ENCODED_AUDIO_TS 1
 
-#define MAX_SW_RES_INT (1920 * 1080)
-
 void DShowInput::OnEncodedVideoData(enum AVCodecID id, unsigned char *data,
 				    size_t size, long long ts)
 {
-	/* If format changes, free and allow it to recreate the decoder */
+	/* If format or hw decode changes, recreate the decoder */
 	if (ffmpeg_decode_valid(video_decoder) &&
-	    video_decoder->codec->id != id) {
+	    ((video_decoder->codec->id != id) ||
+	     (video_decoder->hw != hw_decode))) {
 		ffmpeg_decode_free(video_decoder);
 	}
 
 	if (!ffmpeg_decode_valid(video_decoder)) {
-		/* Only use MJPEG hardware decoding on resolutions higher
-		 * than 1920x1080.  The reason why is because we want to strike
-		 * a reasonable balance between hardware and CPU usage. */
-		bool useHW = videoConfig.format != VideoFormat::MJPEG ||
-			     (videoConfig.cx * videoConfig.cy_abs) >
-				     MAX_SW_RES_INT;
-		if (ffmpeg_decode_init(video_decoder, id, useHW) < 0) {
+		if (ffmpeg_decode_init(video_decoder, id, hw_decode) < 0) {
 			blog(LOG_WARNING, "Could not initialize video decoder");
 			return;
 		}
@@ -504,7 +506,7 @@ void DShowInput::OnEncodedVideoData(enum AVCodecID id, unsigned char *data,
 
 	bool got_output;
 	bool success = ffmpeg_decode_video(video_decoder, data, size, &ts,
-					   range, &frame, &got_output);
+					   frame.range, &frame, &got_output);
 	if (!success) {
 		blog(LOG_WARNING, "Error decoding video");
 		return;
@@ -521,6 +523,11 @@ void DShowInput::OnEncodedVideoData(enum AVCodecID id, unsigned char *data,
 	}
 }
 
+void DShowInput::OnReactivate()
+{
+	SetActive(true);
+}
+
 void DShowInput::OnVideoData(const VideoConfig &config, unsigned char *data,
 			     size_t size, long long startTime,
 			     long long endTime, long rotation)
@@ -535,6 +542,13 @@ void DShowInput::OnVideoData(const VideoConfig &config, unsigned char *data,
 		return;
 	}
 
+#ifdef ENABLE_HEVC
+	if (videoConfig.format == VideoFormat::HEVC) {
+		OnEncodedVideoData(AV_CODEC_ID_HEVC, data, size, startTime);
+		return;
+	}
+#endif
+
 	if (videoConfig.format == VideoFormat::MJPEG) {
 		OnEncodedVideoData(AV_CODEC_ID_MJPEG, data, size, startTime);
 		return;
@@ -596,6 +610,12 @@ void DShowInput::OnVideoData(const VideoConfig &config, unsigned char *data,
 		frame.data[0] = data;
 		frame.linesize[0] = cx;
 
+	} else if (videoConfig.format == VideoFormat::P010) {
+		frame.data[0] = data;
+		frame.data[1] = frame.data[0] + (cx * cy_abs) * 2;
+		frame.linesize[0] = cx * 2;
+		frame.linesize[1] = cx * 2;
+
 	} else {
 		/* TODO: other formats */
 		return;
@@ -604,7 +624,6 @@ void DShowInput::OnVideoData(const VideoConfig &config, unsigned char *data,
 	obs_source_output_video2(source, &frame);
 
 	UNUSED_PARAMETER(endTime); /* it's the enndd tiimmes! */
-	UNUSED_PARAMETER(size);
 }
 
 void DShowInput::OnEncodedAudioData(enum AVCodecID id, unsigned char *data,
@@ -874,6 +893,7 @@ bool DShowInput::UpdateVideoConfig(obs_data_t *settings)
 	deactivateWhenNotShowing = obs_data_get_bool(settings, DEACTIVATE_WNS);
 	flip = obs_data_get_bool(settings, FLIP_IMAGE);
 	autorotation = obs_data_get_bool(settings, AUTOROTATION);
+	hw_decode = obs_data_get_bool(settings, HW_DECODE);
 
 	DeviceId id;
 	if (!DecodeDeviceId(id, video_device_id.c_str())) {
@@ -950,6 +970,8 @@ bool DShowInput::UpdateVideoConfig(obs_data_t *settings)
 					 placeholders::_1, placeholders::_2,
 					 placeholders::_3, placeholders::_4,
 					 placeholders::_5, placeholders::_6);
+	videoConfig.reactivateCallback =
+		std::bind(&DShowInput::OnReactivate, this);
 
 	videoConfig.format = videoConfig.internalFormat;
 
@@ -984,12 +1006,14 @@ bool DShowInput::UpdateVideoConfig(obs_data_t *settings)
 	     "\tflip: %d\n"
 	     "\tfps: %0.2f (interval: %lld)\n"
 	     "\tformat: %s\n"
-	     "\tbuffering: %s",
+	     "\tbuffering: %s\n"
+	     "\thardware decode: %s",
 	     obs_source_get_name(source), (const char *)name_utf8,
 	     (const char *)path_utf8, videoConfig.cx, videoConfig.cy_abs,
 	     (int)videoConfig.cy_flip, fps, videoConfig.frameInterval,
 	     formatName->array,
-	     obs_source_async_unbuffered(source) ? "disabled" : "enabled");
+	     obs_source_async_unbuffered(source) ? "disabled" : "enabled",
+	     hw_decode ? "enabled" : "disabled");
 
 	return true;
 }
@@ -1084,6 +1108,15 @@ DShowInput::GetColorSpace(obs_data_t *settings) const
 	if (astrcmpi(space, "601") == 0)
 		return VIDEO_CS_601;
 
+	if (astrcmpi(space, "2100PQ") == 0)
+		return VIDEO_CS_2100_PQ;
+
+	if (astrcmpi(space, "2100HLG") == 0)
+		return VIDEO_CS_2100_HLG;
+
+	if (videoConfig.format == VideoFormat::P010)
+		return VIDEO_CS_2100_PQ;
+
 	return VIDEO_CS_DEFAULT;
 }
 
@@ -1122,14 +1155,31 @@ inline bool DShowInput::Activate(obs_data_t *settings)
 	if (device.Start() != Result::Success)
 		return false;
 
-	enum video_colorspace cs = GetColorSpace(settings);
-	range = GetColorRange(settings);
-	frame.range = range;
+	const enum video_colorspace cs = GetColorSpace(settings);
+	const enum video_range_type range = GetColorRange(settings);
 
-	bool success = video_format_get_parameters(cs, range,
-						   frame.color_matrix,
-						   frame.color_range_min,
-						   frame.color_range_max);
+	enum video_trc trc = VIDEO_TRC_DEFAULT;
+	switch (cs) {
+	case VIDEO_CS_DEFAULT:
+	case VIDEO_CS_601:
+	case VIDEO_CS_709:
+	case VIDEO_CS_SRGB:
+		trc = VIDEO_TRC_SRGB;
+		break;
+	case VIDEO_CS_2100_PQ:
+		trc = VIDEO_TRC_PQ;
+		break;
+	case VIDEO_CS_2100_HLG:
+		trc = VIDEO_TRC_HLG;
+	}
+
+	frame.range = range;
+	frame.trc = trc;
+
+	bool success = video_format_get_parameters_for_format(
+		cs, range, ConvertVideoFormat(videoConfig.format),
+		frame.color_matrix, frame.color_range_min,
+		frame.color_range_max);
 	if (!success) {
 		blog(LOG_ERROR,
 		     "Failed to get video format parameters for "
@@ -1200,6 +1250,7 @@ static void GetDShowDefaults(obs_data_t *settings)
 	obs_data_set_default_int(settings, AUDIO_OUTPUT_MODE,
 				 (int)AudioMode::Capture);
 	obs_data_set_default_bool(settings, AUTOROTATION, true);
+	obs_data_set_default_bool(settings, HW_DECODE, false);
 }
 
 struct Resolution {
@@ -1348,6 +1399,7 @@ static const VideoFormatName videoFormatNames[] = {
 	{VideoFormat::NV12, "NV12"},
 	{VideoFormat::YV12, "YV12"},
 	{VideoFormat::Y800, "Y800"},
+	{VideoFormat::P010, "P010"},
 
 	/* packed YUV formats */
 	{VideoFormat::YVYU, "YVYU"},
@@ -1357,7 +1409,11 @@ static const VideoFormatName videoFormatNames[] = {
 
 	/* encoded formats */
 	{VideoFormat::MJPEG, "MJPEG"},
-	{VideoFormat::H264, "H264"}};
+	{VideoFormat::H264, "H264"},
+#ifdef ENABLE_HEVC
+	{VideoFormat::HEVC, "HEVC"},
+#endif
+};
 
 static bool ResTypeChanged(obs_properties_t *props, obs_property_t *p,
 			   obs_data_t *settings);
@@ -1775,7 +1831,6 @@ static bool DeviceIntervalChanged(obs_properties_t *props, obs_property_t *p,
 	UpdateVideoFormats(device, format, cx, cy, val, props);
 	UpdateFPS(device, format, val, cx, cy, props);
 
-	UNUSED_PARAMETER(p);
 	return true;
 }
 
@@ -1925,8 +1980,10 @@ static obs_properties_t *GetDShowProperties(void *obj)
 				    OBS_COMBO_TYPE_LIST,
 				    OBS_COMBO_FORMAT_STRING);
 	obs_property_list_add_string(p, TEXT_COLOR_DEFAULT, "default");
-	obs_property_list_add_string(p, "709", "709");
-	obs_property_list_add_string(p, "601", "601");
+	obs_property_list_add_string(p, TEXT_COLOR_709, "709");
+	obs_property_list_add_string(p, TEXT_COLOR_601, "601");
+	obs_property_list_add_string(p, TEXT_COLOR_2100PQ, "2100PQ");
+	obs_property_list_add_string(p, TEXT_COLOR_2100HLG, "2100HLG");
 
 	p = obs_properties_add_list(ppts, COLOR_RANGE, TEXT_COLOR_RANGE,
 				    OBS_COMBO_TYPE_LIST,
@@ -1951,6 +2008,8 @@ static obs_properties_t *GetDShowProperties(void *obj)
 
 	obs_properties_add_bool(ppts, AUTOROTATION, TEXT_AUTOROTATION);
 
+	obs_properties_add_bool(ppts, HW_DECODE, TEXT_HW_DECODE);
+
 	/* ------------------------------------- */
 	/* audio settings */
 
